import functools
import pytest

from devtools_testutils import AzureRecordedTestCase, EnvironmentVariableLoader, recorded_by_proxy
from azure.ai.language.conversations.aio import ConversationAnalysisClient
from azure.ai.language.conversations.models import (
    AnalyzeConversationOperationInput,
    ConversationActionContent,
    ConversationAnalysisInput,
    TextConversationItem,
    ConversationActionResult,
    ConversationPrediction,
    ConversationIntent,
    ConversationEntity,
    StringIndexType,
    ResolutionBase,
    DateTimeResolution,
    ConversationLanguageUnderstandingInput,
)
from typing import cast
from devtools_testutils.aio import recorded_by_proxy_async
from azure.core.async_paging import AsyncItemPaged

from azure.core.credentials import AzureKeyCredential

ConversationsPreparer = functools.partial(
    EnvironmentVariableLoader,
    "conversations",
    conversations_endpoint="https://Sanitized.cognitiveservices.azure.com/",
    conversations_key="fake_key",
)


class TestConversations(AzureRecordedTestCase):

    # Start with any helper functions you might need, for example a client creation method:
    async def create_client(self, endpoint, key):
        credential = AzureKeyCredential(key)
        client = ConversationAnalysisClient(endpoint, credential)
        return client

    ...


class TestConversationsCase(TestConversations):
    @ConversationsPreparer()
    @recorded_by_proxy_async
    @pytest.mark.asyncio
    async def test_conversation_prediction_with_options_async(self, conversations_endpoint, conversations_key):
        client = await self.create_client(conversations_endpoint, conversations_key)

        try:
            project_name = "EmailApp"
            deployment_name = "production"

            data = ConversationLanguageUnderstandingInput(
                conversation_input=ConversationAnalysisInput(
                    conversation_item=TextConversationItem(
                        id="1",
                        participant_id="participant1",
                        text="Send an email to Carol about tomorrow's demo",
                    )
                ),
                action_content=ConversationActionContent(
                    project_name=project_name,
                    deployment_name=deployment_name,
                    string_index_type=StringIndexType.UTF16_CODE_UNIT,
                    verbose=True,
                ),
            )

            action_result = await client.analyze_conversation(data)
            action_result = cast(ConversationActionResult, action_result)

            prediction = action_result.result.prediction
            prediction = cast(ConversationPrediction, prediction)

            print(f"Top intent: {prediction.top_intent}")

            print("Intents:")
            for intent in prediction.intents:
                print(f"Category: {intent.category}")
                print(f"Confidence: {intent.confidence}")
                print()

            print("Entities:")
            for entity in prediction.entities:
                print(f"Category: {entity.category}")
                print(f"Text: {entity.text}")
                print(f"Offset: {entity.offset}")
                print(f"Length: {entity.length}")
                print(f"Confidence: {entity.confidence}")
                print()

                if entity.resolutions:
                    for res in entity.resolutions:
                        # type-safe: only access attrs if it's a DateTimeResolution
                        if isinstance(res, DateTimeResolution):
                            print(f"Datetime Sub Kind: {res.date_time_sub_kind}")
                            print(f"Timex: {res.timex}")
                            print(f"Value: {res.value}")
                            print()

            assert prediction.top_intent == "SendEmail"
        finally:
            await client.close()



# pylint: disable=line-too-long,useless-suppression
import functools
import pytest

from devtools_testutils import AzureRecordedTestCase, EnvironmentVariableLoader
from devtools_testutils.aio import recorded_by_proxy_async
from azure.core.async_paging import AsyncItemPaged
from azure.ai.language.conversations.aio import ConversationAnalysisClient, AnalyzeConversationAsyncLROPoller
from azure.ai.language.conversations.models import (
    AnalyzeConversationActionResult,
    ConversationActionContent,
    ConversationAnalysisInput,
    TextConversationItem,
    StringIndexType,
    ConversationLanguageUnderstandingInput,
    OrchestrationPrediction,
    QuestionAnsweringTargetIntentResult,
    ConversationActionResult,
)
from typing import cast, List
from azure.core.credentials import AzureKeyCredential

ConversationsPreparer = functools.partial(
    EnvironmentVariableLoader,
    "conversations",
    conversations_endpoint="https://Sanitized.cognitiveservices.azure.com/",
    conversations_key="fake_key",
)


class TestConversations(AzureRecordedTestCase):
    # Async client factory
    async def create_async_client(self, endpoint: str, key: str) -> ConversationAnalysisClient:
        credential = AzureKeyCredential(key)
        return ConversationAnalysisClient(endpoint, credential)


class TestConversationsCase(TestConversations):
    @ConversationsPreparer()
    @recorded_by_proxy_async
    @pytest.mark.asyncio
    async def test_orchestration_prediction_async(self, conversations_endpoint, conversations_key):
        client = await self.create_async_client(conversations_endpoint, conversations_key)
        try:
            project_name = "TestWorkflow"
            deployment_name = "production"

            # Build request using strongly-typed models
            data = ConversationLanguageUnderstandingInput(
                conversation_input=ConversationAnalysisInput(
                    conversation_item=TextConversationItem(
                        id="1",
                        participant_id="participant1",
                        text="How are you?",
                    )
                ),
                action_content=ConversationActionContent(
                    project_name=project_name,
                    deployment_name=deployment_name,
                    string_index_type=StringIndexType.UTF16_CODE_UNIT,
                ),
            )

            # Call async API
            response: AnalyzeConversationActionResult = await client.analyze_conversation(data)

            # Cast to the specific discriminator subtype (C#-style)
            conversation_result = cast(ConversationActionResult, response)
            prediction = conversation_result.result.prediction

            assert isinstance(prediction, OrchestrationPrediction)

            # top_intent is Optional[str] in the model; guard to satisfy type checker
            assert prediction.top_intent is not None, "top_intent missing in orchestration prediction"
            responding_project_name = cast(str, prediction.top_intent)
            print(f"Top intent: {responding_project_name}")

            target_intent_result = prediction.intents[responding_project_name]
            assert isinstance(target_intent_result, QuestionAnsweringTargetIntentResult)

            qa = target_intent_result.result
            for ans in qa.answers:  # type: ignore
                print(ans.answer or "")

            # Final assertions like in C#
            assert responding_project_name == "ChitChat-QnA"
        finally:
            await client.close()



# pylint: disable=line-too-long,useless-suppression
# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------
"""Customize generated code here.

Follow our quickstart for examples: https://aka.ms/azsdk/python/dpcodegen/python/customize
"""
import json
from typing import Any, Callable, Dict, IO, Mapping, Optional, TypeVar, Union, cast, overload, Generic, TYPE_CHECKING
from collections.abc import MutableMapping  # pylint:disable=import-error
from urllib.parse import urlparse
from azure.core.exceptions import (
    HttpResponseError,
)
from azure.core.pipeline import PipelineResponse
from azure.core.polling import AsyncLROPoller, AsyncNoPolling, AsyncPollingMethod
from azure.core.polling.async_base_polling import AsyncLROBasePolling
from azure.core.rest import AsyncHttpResponse, HttpRequest
from azure.core.tracing.decorator_async import distributed_trace_async
from azure.core.utils import case_insensitive_dict
from azure.core.credentials import AzureKeyCredential
from azure.core.async_paging import AsyncItemPaged
from ._client import ConversationAnalysisClient as AnalysisClientGenerated
from ..models import AnalyzeConversationOperationInput, AnalyzeConversationOperationState, ConversationActions

from .._validation import api_version_validation

if TYPE_CHECKING:
    from azure.core.credentials_async import AsyncTokenCredential

JSON = MutableMapping[str, Any]
T = TypeVar("T")
ClsType = Optional[Callable[[PipelineResponse[HttpRequest, AsyncHttpResponse], T, Dict[str, Any]], Any]]


def _parse_operation_id(op_loc: Optional[str]) -> Optional[str]:
    """Extract the operation ID from an Operation-Location URL.

    :param op_loc: The ``Operation-Location`` header value or URL to parse.
        If ``None`` or malformed, no ID can be extracted.
    :type op_loc: Optional[str]
    :return: The trailing path segment as the operation ID, or ``None`` if not found.
    :rtype: Optional[str]
    """
    if not op_loc:
        return None
    path = urlparse(op_loc).path.rstrip("/")
    if "/" not in path:
        return None
    return path.rsplit("/", 1)[-1]


PollingReturnType_co = TypeVar("PollingReturnType_co", covariant=True)


class AnalyzeConversationAsyncLROPoller(AsyncLROPoller[PollingReturnType_co], Generic[PollingReturnType_co]):
    """Async poller that returns PollingReturnType_co and exposes operation metadata."""

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)
        # populated by your deserialization callback in begin_*_async
        self._last_state: Optional["AnalyzeConversationOperationState"] = None

    def _record_state_for_details(self, state: "AnalyzeConversationOperationState") -> None:
        """Internal: update the state used by ``.details``.

        :param state: The latest operation state to expose via ``details``.
        :type state: AnalyzeConversationOperationState
        :return: None
        :rtype: None
        """
        self._last_state = state

    @property
    def details(self) -> Mapping[str, Any]:
        """Metadata associated with the long-running operation.

        :return: A mapping with keys like ``operation_id`` and, when available,
            ``status``, ``job_id``, ``display_name``, ``created_date_time``,
            ``last_updated_date_time``, ``expiration_date_time``, ``statistics``,
            ``errors``, and ``next_link``.
        :rtype: Mapping[str, Any]
        """
        try:
            headers = getattr(self.polling_method(), "_initial_response").http_response.headers  # type: ignore[attr-defined]
            op_loc = headers.get("Operation-Location") or headers.get("operation-location")
        except (AttributeError, TypeError):
            # missing attributes in the chain, or headers is not a mapping
            op_loc = None

        info: Dict[str, Any] = {"operation_id": _parse_operation_id(op_loc)}

        # Enrich from final state if available
        if self._last_state is not None:
            s = self._last_state
            info.update(
                {
                    "status": s.status,
                    "job_id": s.job_id,
                    "display_name": s.display_name,
                    "created_date_time": s.created_date_time,
                    "last_updated_date_time": s.last_updated_date_time,
                    "expiration_date_time": s.expiration_date_time,
                    "statistics": s.statistics,
                    "errors": s.errors,
                    "next_link": s.next_link,
                }
            )
        return info

    @classmethod
    def from_continuation_token(
        cls,
        polling_method: AsyncPollingMethod[PollingReturnType_co],
        continuation_token: str,
        **kwargs: Any,
    ) -> "AnalyzeConversationAsyncLROPoller[PollingReturnType_co]":
        client, initial_response, deserialization_callback = polling_method.from_continuation_token(
            continuation_token, **kwargs
        )
        return cls(client, initial_response, deserialization_callback, polling_method)


class ConversationAnalysisClient(AnalysisClientGenerated):

    def __init__(
        self,
        endpoint: str,
        credential: Union[AzureKeyCredential, "AsyncTokenCredential"],
        *,
        api_version: Optional[str] = None,
        **kwargs: Any,
    ) -> None:
        """Create a ConversationAnalysisClient.
        :param endpoint: Supported Cognitive Services endpoint.
        :type endpoint: str
        :param credential: Key or token credential.
        :type credential: ~azure.core.credentials.AzureKeyCredential or ~azure.core.credentials.TokenCredential
        :keyword api_version: The API version to use for this operation. Default value is
        "2025-11-15-preview". Note that overriding this default value may result in unsupported
        behavior.
        :paramtype api_version: str`
        """
        if api_version is not None:
            kwargs["api_version"] = api_version
        super().__init__(endpoint=endpoint, credential=credential, **kwargs)

    @overload
    async def begin_analyze_conversation_job(
        self, body: AnalyzeConversationOperationInput, *, content_type: str = "application/json", **kwargs: Any
    ) -> AnalyzeConversationAsyncLROPoller[AsyncItemPaged["ConversationActions"]]:
        """Analyzes the input conversation utterance.

        :param body: The input for the analyze conversations operation. Required.
        :type body: ~azure.ai.language.conversations.models.AnalyzeConversationOperationInput
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: An async poller whose ``result()`` yields ``AsyncItemPaged[ConversationActions]``; exposes metadata via ``.details``.
        :rtype: ~azure.ai.language.conversations.AnalyzeConversationAsyncLROPoller[
                ~azure.core.async_paging.AsyncItemPaged[~azure.ai.language.conversations.models.ConversationActions]]
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    async def begin_analyze_conversation_job(
        self, body: JSON, *, content_type: str = "application/json", **kwargs: Any
    ) -> AnalyzeConversationAsyncLROPoller[AsyncItemPaged["ConversationActions"]]:
        """Analyzes the input conversation utterance.

        :param body: The input for the analyze conversations operation. Required.
        :type body: JSON
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: An async poller whose ``result()`` yields ``AsyncItemPaged[ConversationActions]``; exposes metadata via ``.details``.
        :rtype: ~azure.ai.language.conversations.AnalyzeConversationAsyncLROPoller[
                ~azure.core.async_paging.AsyncItemPaged[~azure.ai.language.conversations.models.ConversationActions]]
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    async def begin_analyze_conversation_job(
        self, body: IO[bytes], *, content_type: str = "application/json", **kwargs: Any
    ) -> AnalyzeConversationAsyncLROPoller[AsyncItemPaged["ConversationActions"]]:
        """Analyzes the input conversation utterance.

        :param body: The input for the analyze conversations operation. Required.
        :type body: IO[bytes]
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: An async poller whose ``result()`` yields ``AsyncItemPaged[ConversationActions]``; exposes metadata via ``.details``.
        :rtype: ~azure.ai.language.conversations.AnalyzeConversationAsyncLROPoller[
                ~azure.core.async_paging.AsyncItemPaged[~azure.ai.language.conversations.models.ConversationActions]]
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace_async
    @api_version_validation(
        method_added_on="2023-04-01",
        params_added_on={"2023-04-01": ["api_version", "content_type", "accept"]},
        api_versions_list=["2023-04-01", "2024-05-01", "2024-11-01", "2025-05-15-preview", "2025-11-15-preview"],
    )
    async def begin_analyze_conversation_job(  # type: ignore[override]
        self, body: Union[AnalyzeConversationOperationInput, JSON, IO[bytes]], **kwargs: Any
    ) -> AnalyzeConversationAsyncLROPoller[AsyncItemPaged["ConversationActions"]]:
        """Analyzes the input conversation utterance.

        :param body: The input for the analyze conversations operation. Required.
        :type body: ~azure.ai.language.conversations.models.AnalyzeConversationOperationInput or JSON or IO[bytes]
        :return: An async poller whose ``result()`` yields ``AsyncItemPaged[ConversationActions]``; exposes metadata via ``.details``.
        :rtype: ~azure.ai.language.conversations.AnalyzeConversationAsyncLROPoller[
                ~azure.core.async_paging.AsyncItemPaged[~azure.ai.language.conversations.models.ConversationActions]]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        polling: Union[bool, AsyncPollingMethod[AsyncItemPaged["ConversationActions"]]] = kwargs.pop("polling", True)
        lro_delay = kwargs.pop("polling_interval", self._config.polling_interval)
        cont_token: Optional[str] = kwargs.pop("continuation_token", None)
        cls = kwargs.pop("cls", None)  # optional custom deserializer
        kwargs.pop("error_map", None)

        path_format_arguments = {
            "Endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }

        # ---- paging helpers (state -> AsyncItemPaged[ConversationActions])

        async def _fetch_state_by_next_link(next_link: str) -> AnalyzeConversationOperationState:
            req = HttpRequest("GET", next_link)
            resp = await self._client.send_request(req)  # type: ignore[attr-defined]
            if resp.status_code != 200:
                raise HttpResponseError(response=resp)
            await resp.read()
            data = json.loads(resp.text())
            return AnalyzeConversationOperationState(data)

        def _build_pager_from_state(state: AnalyzeConversationOperationState) -> AsyncItemPaged["ConversationActions"]:
            async def extract_data(s: AnalyzeConversationOperationState):
                next_link = s.next_link
                actions: ConversationActions = s.actions
                return next_link, [actions]

            async def get_next(token: Optional[str]) -> Optional[AnalyzeConversationOperationState]:
                if token is None:
                    return state
                if not token:
                    return None
                return await _fetch_state_by_next_link(token)

            return AsyncItemPaged(get_next, extract_data)

        # holder to let the deserializer set poller._last_state
        poller_holder: Dict[str, AnalyzeConversationAsyncLROPoller[AsyncItemPaged["ConversationActions"]]] = {}

        # ---- deserializer: final HTTP -> AsyncItemPaged[ConversationActions]
        def get_long_running_output(pipeline_response):
            final = pipeline_response.http_response
            if final.status_code == 200:
                data = json.loads(final.text())
                op_state = AnalyzeConversationOperationState(data)

                poller_ref = poller_holder["poller"]
                poller_ref._record_state_for_details(op_state) # pylint:disable=protected-access

                paged = _build_pager_from_state(op_state)
                return cls(pipeline_response, paged, {}) if cls else paged
            raise HttpResponseError(response=final)

        # ---- polling method
        if polling is True:
            polling_method: AsyncPollingMethod[AsyncItemPaged["ConversationActions"]] = cast(
                AsyncPollingMethod[AsyncItemPaged["ConversationActions"]],
                AsyncLROBasePolling(lro_delay, path_format_arguments=path_format_arguments, **kwargs),
            )
        elif polling is False:
            polling_method = cast(AsyncPollingMethod[AsyncItemPaged["ConversationActions"]], AsyncNoPolling())
        else:
            polling_method = cast(AsyncPollingMethod[AsyncItemPaged["ConversationActions"]], polling)

        # ---- resume path
        if cont_token:
            return AnalyzeConversationAsyncLROPoller[AsyncItemPaged["ConversationActions"]].from_continuation_token(
                polling_method=polling_method,
                continuation_token=cont_token,
            )

        # ---- submit job
        raw_result = await self._analyze_conversation_job_initial(
            body=body,
            content_type=content_type,
            cls=lambda x, y, z: x,  # passthrough
            headers=_headers,
            params=_params,
            **kwargs,
        )
        # buffer initial body so .text() is available later
        await raw_result.http_response.read()  # type: ignore[attr-defined]

        # ---- build custom async poller
        lro: AnalyzeConversationAsyncLROPoller[AsyncItemPaged["ConversationActions"]] = (
            AnalyzeConversationAsyncLROPoller(self._client, raw_result, get_long_running_output, polling_method)
        )
        poller_holder["poller"] = lro
        return lro


def patch_sdk():
    """Do not remove from this file.

    `patch_sdk` is a last resort escape hatch that allows you to do customizations
    you can't accomplish using the techniques described in
    https://aka.ms/azsdk/python/dpcodegen/python/customize
    """


__all__ = ["ConversationAnalysisClient", "AnalyzeConversationAsyncLROPoller"]



import functools
import pytest

from devtools_testutils import AzureRecordedTestCase, EnvironmentVariableLoader, recorded_by_proxy
from azure.ai.language.conversations import ConversationAnalysisClient, AnalyzeConversationLROPoller
from azure.core.paging import ItemPaged
from azure.ai.language.conversations.models import (
    # request models
    AnalyzeConversationOperationInput,
    MultiLanguageConversationInput,
    TextConversation,
    TextConversationItem,
    PiiOperationAction,
    ConversationPiiActionContent,
    ConversationActions,
    AnalyzeConversationOperationResult,
    ConversationPiiOperationResult,
    ConversationalPiiResult,
    ConversationPiiItemResult,
    NamedEntity,
    InputWarning,
    ConversationError,
    AnalyzeConversationOperationAction,
)
from typing import cast, List

from azure.core.credentials import AzureKeyCredential

ConversationsPreparer = functools.partial(
    EnvironmentVariableLoader,
    "conversations",
    conversations_endpoint="https://Sanitized.cognitiveservices.azure.com/",
    conversations_key="fake_key",
)


class TestConversations(AzureRecordedTestCase):

    # Start with any helper functions you might need, for example a client creation method:
    def create_client(self, endpoint, key):
        credential = AzureKeyCredential(key)
        client = ConversationAnalysisClient(endpoint, credential)
        return client

    ...


class TestConversationsCase(TestConversations):
    @ConversationsPreparer()
    @recorded_by_proxy
    def test_conversation_pii(self, conversations_endpoint, conversations_key):
        client = self.create_client(conversations_endpoint, conversations_key)

        entities_detected: List[NamedEntity] = []

        # ---- Build input ------------------------------------
        ml_input = MultiLanguageConversationInput(
            conversations=[
                TextConversation(
                    id="1",
                    language="en",
                    conversation_items=[
                        TextConversationItem(id="1", participant_id="Agent_1", text="Can you provide you name?"),
                        TextConversationItem(id="2", participant_id="Customer_1", text="Hi, my name is John Doe."),
                        TextConversationItem(
                            id="3",
                            participant_id="Agent_1",
                            text="Thank you John, that has been updated in our system.",
                        ),
                    ],
                )
            ]
        )

        pii_action: AnalyzeConversationOperationAction = PiiOperationAction(
            action_content=ConversationPiiActionContent(),
            name="Conversation PII",
        )
        actions: List[AnalyzeConversationOperationAction] = [pii_action]

        operation_input = AnalyzeConversationOperationInput(
            conversation_input=ml_input,
            actions=actions,
        )

        # ---- Begin LRO --------------------------------------------------------
        poller: AnalyzeConversationLROPoller[ItemPaged[ConversationActions]] = client.begin_analyze_conversation_job(
            body=operation_input
        )

        # Operation metadata is available immediately
        print(f"Operation ID: {poller.details.get('operation_id')}")

        # Wait for completion; result is ItemPaged[ConversationActions]
        paged_actions: ItemPaged[ConversationActions] = poller.result()

        # Final-state metadata
        d = poller.details
        print(f"Job ID: {d.get('job_id')}")
        print(f"Status: {d.get('status')}")
        print(f"Created: {d.get('created_date_time')}")
        print(f"Last Updated: {d.get('last_updated_date_time')}")
        if d.get("expiration_date_time"):
            print(f"Expires: {d.get('expiration_date_time')}")
        if d.get("display_name"):
            print(f"Display Name: {d.get('display_name')}")

        # ---- Iterate pages and action results --------------------------------
        for actions_page in paged_actions:
            print(
                f"Completed: {actions_page.completed}, "
                f"In Progress: {actions_page.in_progress}, "
                f"Failed: {actions_page.failed}, "
                f"Total: {actions_page.total}"
            )

            for action_result in actions_page.task_results or []:
                ar = cast(AnalyzeConversationOperationResult, action_result)
                print(f"\nAction Name: {getattr(ar, 'name', None)}")
                print(f"Action Status: {getattr(ar, 'status', None)}")
                print(f"Kind: {getattr(ar, 'kind', None)}")

                if isinstance(ar, ConversationPiiOperationResult):
                    for conversation in ar.results.conversations or []:
                        conversation = cast(ConversationalPiiResult, conversation)
                        print(f"Conversation: #{conversation.id}")
                        print("Detected Entities:")
                        for item in conversation.conversation_items or []:
                            item = cast(ConversationPiiItemResult, item)
                            for entity in item.entities or []:
                                entity = cast(NamedEntity, entity)
                                print(f"  Category: {entity.category}")
                                print(f"  Subcategory: {entity.subcategory}")
                                print(f"  Text: {entity.text}")
                                print(f"  Offset: {entity.offset}")
                                print(f"  Length: {entity.length}")
                                print(f"  Confidence score: {entity.confidence_score}\n")
                                entities_detected.append(entity)

                        if conversation.warnings:
                            print("Warnings:")
                            for warning in conversation.warnings:
                                warning = cast(InputWarning, warning)
                                print(f"  Code: {warning.code}")
                                print(f"  Message: {warning.message}")
                        print()
                else:
                    print("  [No supported results to display for this action type]")

        # ---- Print errors (from final-state metadata) -------------------------
        if d.get("errors"):
            print("\nErrors:")
            for err in d["errors"]:
                err = cast(ConversationError, err)
                print(f"  Code: {err.code} - {err.message}")

        # ---- Assertions -------------------------------------------------------
        assert len(entities_detected) > 0, "Expected at least one PII entity."
        assert (d.get("status") or "").lower() in {"succeeded", "partiallysucceeded"}



# pylint: disable=line-too-long,useless-suppression
import functools
import pytest

from devtools_testutils import AzureRecordedTestCase, EnvironmentVariableLoader, recorded_by_proxy
from azure.ai.language.conversations import ConversationAnalysisClient, AnalyzeConversationLROPoller
from azure.core.paging import ItemPaged
from azure.ai.language.conversations.models import (
    # request models
    AnalyzeConversationOperationInput,
    MultiLanguageConversationInput,
    TextConversation,
    TextConversationItem,
    PiiOperationAction,
    ConversationPiiActionContent,
    ConversationActions,
    AnalyzeConversationOperationResult,
    ConversationPiiOperationResult,
    ConversationalPiiResult,
    ConversationPiiItemResult,
    NamedEntity,
    InputWarning,
    ConversationError,
    AnalyzeConversationOperationAction,
    CharacterMaskPolicyType,
    RedactionCharacter,
    EntityMaskTypePolicyType,
)
from typing import cast, List
import re

from azure.core.credentials import AzureKeyCredential

ConversationsPreparer = functools.partial(
    EnvironmentVariableLoader,
    "conversations",
    conversations_endpoint="https://Sanitized.azure-api.net/",
    conversations_key="fake_key",
)


class TestConversations(AzureRecordedTestCase):

    # Start with any helper functions you might need, for example a client creation method:
    def create_client(self, endpoint, key):
        credential = AzureKeyCredential(key)
        client = ConversationAnalysisClient(endpoint, credential)
        return client

    ...


class TestConversationsCase(TestConversations):
    @ConversationsPreparer()
    @recorded_by_proxy
    def test_conversation_pii_with_entity_mask_policy(self, conversations_endpoint, conversations_key):
        client = self.create_client(conversations_endpoint, conversations_key)

        # Track redacted texts we verify
        redacted_verified: List[str] = []

        # ---- Redaction policy: mask entities as [Category] / [Category-1] ----
        redaction_policy = EntityMaskTypePolicyType()

        # ---- Build input -----------------------------------------------------
        ml_input = MultiLanguageConversationInput(
            conversations=[
                TextConversation(
                    id="1",
                    language="en",
                    conversation_items=[
                        TextConversationItem(id="1", participant_id="Agent_1", text="Can you provide your name?"),
                        TextConversationItem(id="2", participant_id="Customer_1", text="Hi, my name is John Doe."),
                        TextConversationItem(
                            id="3",
                            participant_id="Agent_1",
                            text="Thank you John, that has been updated in our system.",
                        ),
                    ],
                )
            ]
        )

        # Action with EntityMaskTypePolicyType
        pii_action: AnalyzeConversationOperationAction = PiiOperationAction(
            action_content=ConversationPiiActionContent(redaction_policy=redaction_policy),
            name="Conversation PII with Entity Mask Policy",
        )
        actions: List[AnalyzeConversationOperationAction] = [pii_action]

        operation_input = AnalyzeConversationOperationInput(
            conversation_input=ml_input,
            actions=actions,
        )

        # ---- Begin LRO -------------------------------------------------------
        poller: AnalyzeConversationLROPoller[ItemPaged[ConversationActions]] = client.begin_analyze_conversation_job(
            body=operation_input
        )

        print(f"Operation ID: {poller.details.get('operation_id')}")

        # Result: ItemPaged[ConversationActions]
        paged_actions: ItemPaged[ConversationActions] = poller.result()

        # Final-state metadata
        d = poller.details
        print(f"Job ID: {d.get('job_id')}")
        print(f"Status: {d.get('status')}")
        if d.get("errors"):
            print("Errors:")
            for err in d["errors"]:
                err = cast(ConversationError, err)
                print(f"  Code: {err.code} - {err.message}")

        # ---- Iterate results and validate redaction --------------------------
        for actions_page in paged_actions:
            for action_result in actions_page.task_results or []:
                ar = cast(AnalyzeConversationOperationResult, action_result)
                print(f"\nAction Name: {getattr(ar, 'name', None)}")

                if isinstance(ar, ConversationPiiOperationResult):
                    for conversation in ar.results.conversations or []:
                        conversation = cast(ConversationalPiiResult, conversation)
                        print(f"Conversation: #{conversation.id}")

                        for item in conversation.conversation_items or []:
                            item = cast(ConversationPiiItemResult, item)
                            redacted_text = (getattr(item.redacted_content, "text", None) or "").strip()
                            print(f"Redacted Text: {redacted_text}")

                            # Only verify when there are detected entities in the original item
                            if item.entities and redacted_text:
                                for entity in item.entities:
                                    entity = cast(NamedEntity, entity)
                                    original_text = entity.text or ""
                                    # 1) original PII should not be present
                                    assert (
                                        original_text not in redacted_text
                                    ), f"Expected entity '{original_text}' to be redacted but found in: {redacted_text}"

                                    # 2) redaction should show an entity mask like [Person] or [Person-1]
                                    expected_mask_pattern = rf"\[{re.escape(entity.category)}-?\d*\]"
                                    assert re.search(expected_mask_pattern, redacted_text, flags=re.IGNORECASE), (
                                        f"Expected redacted text to contain an entity mask similar to "
                                        f"'[{entity.category}]' but got: {redacted_text}"
                                    )

                                redacted_verified.append(redacted_text)

        # ---- Assertions -------------------------------------------------------
        assert (d.get("status") or "").lower() in {"succeeded", "partiallysucceeded"}
        assert len(redacted_verified) > 0, "Expected at least one redacted line to be verified."



import pytest


# autouse=True will trigger this fixture on each pytest run, even if it's not explicitly used by a test method
# test_proxy auto-starts the test proxy
# patch_sleep and patch_async_sleep streamline tests by disabling wait times during LRO polling
@pytest.fixture(scope="session", autouse=True)
def start_proxy(test_proxy, patch_sleep, patch_async_sleep):
    return



# pylint: disable=line-too-long,useless-suppression
import functools
import pytest

from devtools_testutils import AzureRecordedTestCase, EnvironmentVariableLoader, recorded_by_proxy
from azure.ai.language.conversations import ConversationAnalysisClient
from azure.ai.language.conversations.models import (
    ConversationalAITask,
    ConversationalAIAnalysisInput,
    TextConversation,
    TextConversationItem,
    ConversationalAIActionContent,
    AnalyzeConversationActionResult,
    StringIndexType,
    ConversationalAITaskResult,
    ConversationalAIResult,
    ConversationalAIAnalysis,
    ConversationalAIIntent,
    ConversationalAIEntity,
    ConversationItemRange,
    DateTimeResolution,
    EntitySubtype,
    EntityTag,
)
from typing import cast

from azure.core.credentials import AzureKeyCredential

ConversationsPreparer = functools.partial(
    EnvironmentVariableLoader,
    "conversations",
    conversations_endpoint="https://Sanitized.cognitiveservices.azure.com/",
    conversations_key="fake_key",
)


class TestConversations(AzureRecordedTestCase):

    # Start with any helper functions you might need, for example a client creation method:
    def create_client(self, endpoint, key):
        credential = AzureKeyCredential(key)
        client = ConversationAnalysisClient(endpoint, credential)
        return client

    ...


class TestConversationsCase(TestConversations):
    @ConversationsPreparer()
    @recorded_by_proxy
    def test_conversation_multi_turn_prediction(self, conversations_endpoint, conversations_key):
        client = self.create_client(conversations_endpoint, conversations_key)

        project_name = "EmailApp"
        deployment_name = "production"

        data = ConversationalAITask(
            analysis_input=ConversationalAIAnalysisInput(
                conversations=[
                    TextConversation(
                        id="order",
                        language="en-GB",
                        conversation_items=[
                            TextConversationItem(id="1", participant_id="user", text="Hi"),
                            TextConversationItem(id="2", participant_id="bot", text="Hello, how can I help you?"),
                            TextConversationItem(
                                id="3",
                                participant_id="user",
                                text="Send an email to Carol about tomorrow's demo",
                            ),
                        ],
                    )
                ]
            ),
            parameters=ConversationalAIActionContent(
                project_name=project_name,
                deployment_name=deployment_name,
                string_index_type=StringIndexType.UTF16_CODE_UNIT,
            ),
        )

        # Call API
        response: AnalyzeConversationActionResult = client.analyze_conversation(data)

        # Narrow to ConversationalAI task result (C# style: `as ConversationalAITaskResult`)
        ai_task_result = cast(ConversationalAITaskResult, response)
        ai_result: ConversationalAIResult = ai_task_result.result

        # Basic sanity
        assert ai_result is not None
        assert ai_result.conversations is not None

        # Iterate conversations
        for conversation in ai_result.conversations or []:
            conversation = cast(ConversationalAIAnalysis, conversation)
            print(f"Conversation ID: {conversation.id}\n")

            # Intents
            print("Intents:")
            for intent in conversation.intents or []:
                intent = cast(ConversationalAIIntent, intent)
                print(f"  Name: {intent.name}")
                print(f"  Type: {getattr(intent, 'type', None)}")

                print("  Conversation Item Ranges:")
                for rng in intent.conversation_item_ranges or []:
                    rng = cast(ConversationItemRange, rng)
                    print(f"    - Offset: {rng.offset}, Count: {rng.count}")

                print("\n  Entities (Scoped to Intent):")
                for ent in intent.entities or []:
                    ent = cast(ConversationalAIEntity, ent)
                    print(f"    Name: {ent.name}")
                    print(f"    Text: {ent.text}")
                    print(f"    Confidence: {ent.confidence_score}")
                    print(f"    Offset: {ent.offset}, Length: {ent.length}")
                    print(f"    Conversation Item ID: {ent.conversation_item_id}, Index: {ent.conversation_item_index}")

                    # Date/time resolutions
                    if ent.resolutions:
                        for res in ent.resolutions:
                            if isinstance(res, DateTimeResolution):
                                print(
                                    f"    - [DateTimeResolution] SubKind: {getattr(res, 'date_time_sub_kind', None)}, "
                                    f"Timex: {res.timex}, Value: {res.value}"
                                )

                    # Extra information (entity subtype + tags)
                    if ent.extra_information:
                        for extra in ent.extra_information:
                            if isinstance(extra, EntitySubtype):
                                print(f"    - [EntitySubtype] Value: {extra.value}")
                                for tag in extra.tags or []:
                                    tag = cast(EntityTag, tag)
                                    print(f"      • Tag: {tag.name}, Confidence: {tag.confidence_score}")

                    print()

                print()

            # Global entities
            print("Global Entities:")
            for ent in conversation.entities or []:
                ent = cast(ConversationalAIEntity, ent)
                print(f"  Name: {ent.name}")
                print(f"  Text: {ent.text}")
                print(f"  Confidence: {ent.confidence_score}")
                print(f"  Offset: {ent.offset}, Length: {ent.length}")
                print(f"  Conversation Item ID: {ent.conversation_item_id}, Index: {ent.conversation_item_index}")

                if ent.extra_information:
                    for extra in ent.extra_information:
                        if isinstance(extra, EntitySubtype):
                            print(f"    - [EntitySubtype] Value: {extra.value}")
                            for tag in extra.tags or []:
                                tag = cast(EntityTag, tag)
                                print(f"      • Tag: {tag.name}, Confidence: {tag.confidence_score}")
                print()

            print("-" * 40)



import functools
import pytest

from devtools_testutils import AzureRecordedTestCase, EnvironmentVariableLoader, recorded_by_proxy
from azure.ai.language.conversations import ConversationAnalysisClient
from azure.ai.language.conversations.models import (
    ConversationActionContent,
    ConversationAnalysisInput,
    TextConversationItem,
    ConversationActionResult,
    ConversationPrediction,
    StringIndexType,
    DateTimeResolution,
    ConversationLanguageUnderstandingInput,
)
from typing import cast

from azure.core.credentials import AzureKeyCredential

ConversationsPreparer = functools.partial(
    EnvironmentVariableLoader,
    "conversations",
    conversations_endpoint="https://Sanitized.cognitiveservices.azure.com/",
    conversations_key="fake_key",
)


class TestConversations(AzureRecordedTestCase):

    # Start with any helper functions you might need, for example a client creation method:
    def create_client(self, endpoint, key):
        credential = AzureKeyCredential(key)
        client = ConversationAnalysisClient(endpoint, credential)
        return client

    ...


class TestConversationsCase(TestConversations):
    @ConversationsPreparer()
    @recorded_by_proxy
    def test_conversation_prediction(self, conversations_endpoint, conversations_key):
        client = self.create_client(conversations_endpoint, conversations_key)

        project_name = "EmailApp"
        deployment_name = "production"

        data = ConversationLanguageUnderstandingInput(
            conversation_input=ConversationAnalysisInput(
                conversation_item=TextConversationItem(
                    id="1", participant_id="participant1", text="Send an email to Carol about tomorrow's demo"
                )
            ),
            action_content=ConversationActionContent(
                project_name=project_name,
                deployment_name=deployment_name,
                string_index_type=StringIndexType.UTF16_CODE_UNIT,
            ),
        )

        action_result = client.analyze_conversation(data)

        action_result = cast(ConversationActionResult, action_result)

        prediction = action_result.result.prediction
        prediction = cast(ConversationPrediction, prediction)
        print(f"Top intent: {prediction.top_intent}")

        print("Intents:")
        for intent in prediction.intents:
            print(f"Category: {intent.category}")
            print(f"Confidence: {intent.confidence}")
            print()

        print("Entities:")
        for entity in prediction.entities:
            print(f"Category: {entity.category}")
            print(f"Text: {entity.text}")
            print(f"Offset: {entity.offset}")
            print(f"Length: {entity.length}")
            print(f"Confidence: {entity.confidence}")
            print()

            if entity.resolutions:
                for res in entity.resolutions:
                    # type-safe: only access attrs if it's a DateTimeResolution
                    if isinstance(res, DateTimeResolution):
                        print(f"Datetime Sub Kind: {res.date_time_sub_kind}")
                        print(f"Timex: {res.timex}")
                        print(f"Value: {res.value}")
                        print()

        assert prediction.top_intent == "SendEmail"



# coding=utf-8
# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------

"""
FILE: sample_conversation_pii_with_character_mask_policy.py

DESCRIPTION:
    This sample demonstrates how to run a PII detection action over a conversation
    using the `CharacterMaskPolicyType` synchronously. Detected PII is redacted
    by replacing characters with a mask (e.g., '*').

USAGE:
    python sample_conversation_pii_with_character_mask_policy.py

REQUIRED ENV VARS (for AAD / DefaultAzureCredential):
    AZURE_CONVERSATIONS_ENDPOINT
    AZURE_CLIENT_ID
    AZURE_TENANT_ID
    AZURE_CLIENT_SECRET

NOTE:
    If you want to use AzureKeyCredential instead, set:
      - AZURE_CONVERSATIONS_ENDPOINT
      - AZURE_CONVERSATIONS_KEY
"""

# [START conversation_pii_with_character_mask_policy]
import os

from azure.identity import DefaultAzureCredential
from azure.ai.language.conversations import ConversationAnalysisClient
from azure.ai.language.conversations.models import (
    MultiLanguageConversationInput,
    TextConversation,
    TextConversationItem,
    ParticipantRole,
    AnalyzeConversationOperationInput,
    PiiOperationAction,
    ConversationPiiActionContent,
    CharacterMaskPolicyType,
    RedactionCharacter,
    ConversationPiiOperationResult,
    ConversationError,
)


def sample_conversation_pii_with_character_mask_policy():
    # settings
    endpoint = os.environ["AZURE_CONVERSATIONS_ENDPOINT"]

    # AAD credential
    credential = DefaultAzureCredential()

    redacted_verified: list[str] = []

    client = ConversationAnalysisClient(endpoint, credential=credential)

    # build input
    ml_input = MultiLanguageConversationInput(
        conversations=[
            TextConversation(
                id="1",
                language="en",
                conversation_items=[
                    TextConversationItem(
                        id="1",
                        participant_id="Agent_1",
                        role=ParticipantRole.AGENT,
                        text="Can you provide your name?",
                    ),
                    TextConversationItem(
                        id="2",
                        participant_id="Customer_1",
                        role=ParticipantRole.CUSTOMER,
                        text="Hi, my name is John Doe.",
                    ),
                    TextConversationItem(
                        id="3",
                        participant_id="Agent_1",
                        role=ParticipantRole.AGENT,
                        text="Thank you John, that has been updated in our system.",
                    ),
                ],
            )
        ]
    )

    # action with CharacterMaskPolicyType
    redaction_policy = CharacterMaskPolicyType(redaction_character=RedactionCharacter.ASTERISK)
    pii_action = PiiOperationAction(
        action_content=ConversationPiiActionContent(redaction_policy=redaction_policy),
        name="Conversation PII with Character Mask Policy",
    )

    operation_input = AnalyzeConversationOperationInput(
        conversation_input=ml_input,
        actions=[pii_action],
    )

    # start long-running job
    poller = client.begin_analyze_conversation_job(body=operation_input)
    print(f"Operation ID: {poller.details.get('operation_id')}")

    # wait for completion
    paged_actions = poller.result()

    # final metadata
    d = poller.details
    print(f"Job ID: {d.get('job_id')}")
    print(f"Status: {d.get('status')}")
    if d.get("errors"):
        print("Errors:")
        for err in d["errors"]:
            print(f"  Code: {err.code} - {err.message}")

    # iterate results and verify redaction
    for actions_page in paged_actions:
        for action_result in actions_page.task_results or []:
            if isinstance(action_result, ConversationPiiOperationResult):
                for conversation in action_result.results.conversations or []:
                    for item in conversation.conversation_items or []:
                        redacted_text = (item.redacted_content.text or "").strip()
                        if not redacted_text:
                            continue

                        if item.entities:
                            for entity in item.entities:
                                ent_text = entity.text or ""
                                if ent_text in redacted_text:
                                    print(
                                        f"WARNING: Expected '{ent_text}' to be redacted but found in: {redacted_text}"
                                    )

                            if "*" in redacted_text:
                                redacted_verified.append(redacted_text)


# [END conversation_pii_with_character_mask_policy]


def main():
    sample_conversation_pii_with_character_mask_policy()


if __name__ == "__main__":
    main()



# coding=utf-8
# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------

"""
FILE: sample_conversation_pii_async.py

DESCRIPTION:
    This sample demonstrates how to run a PII detection action over a conversation (async).

USAGE:
    python sample_conversation_pii_async.py

REQUIRED ENV VARS (for AAD / DefaultAzureCredential):
    AZURE_CONVERSATIONS_ENDPOINT
    AZURE_CLIENT_ID
    AZURE_TENANT_ID
    AZURE_CLIENT_SECRET

NOTE:
    If you want to use AzureKeyCredential instead, set:
      - AZURE_CONVERSATIONS_ENDPOINT
      - AZURE_CONVERSATIONS_KEY
"""

# [START conversation_pii_async]
import os
import asyncio

from azure.identity.aio import DefaultAzureCredential
from azure.ai.language.conversations.aio import ConversationAnalysisClient
from azure.ai.language.conversations.models import (
    MultiLanguageConversationInput,
    TextConversation,
    TextConversationItem,
    ParticipantRole,
    AnalyzeConversationOperationInput,
    PiiOperationAction,
    ConversationPiiActionContent,
    AnalyzeConversationOperationResult,
    ConversationPiiOperationResult,
    InputWarning,
    ConversationError,
)


async def sample_conversation_pii_async():
    # get settings
    endpoint = os.environ["AZURE_CONVERSATIONS_ENDPOINT"]

    credential = DefaultAzureCredential()

    entities_detected = []

    async with ConversationAnalysisClient(endpoint, credential=credential) as client:
        # build input
        ml_input = MultiLanguageConversationInput(
            conversations=[
                TextConversation(
                    id="1",
                    language="en",
                    conversation_items=[
                        TextConversationItem(
                            id="1",
                            participant_id="Agent_1",
                            role=ParticipantRole.AGENT,
                            text="Can you provide your name?",
                        ),
                        TextConversationItem(
                            id="2",
                            participant_id="Customer_1",
                            role=ParticipantRole.CUSTOMER,
                            text="Hi, my name is John Doe.",
                        ),
                        TextConversationItem(
                            id="3",
                            participant_id="Agent_1",
                            role=ParticipantRole.AGENT,
                            text="Thank you John, that has been updated in our system.",
                        ),
                    ],
                )
            ]
        )

        pii_action = PiiOperationAction(
            action_content=ConversationPiiActionContent(),
            name="Conversation PII",
        )

        operation_input = AnalyzeConversationOperationInput(
            conversation_input=ml_input,
            actions=[pii_action],
        )

        # start async long-running operation
        poller = await client.begin_analyze_conversation_job(body=operation_input)

        # operation metadata
        print(f"Operation ID: {poller.details.get('operation_id')}")

        # wait for completion
        paged_actions = await poller.result()

        # final-state metadata
        d = poller.details
        print(f"Job ID: {d.get('job_id')}")
        print(f"Status: {d.get('status')}")
        print(f"Created: {d.get('created_date_time')}")
        print(f"Last Updated: {d.get('last_updated_date_time')}")
        if d.get("expiration_date_time"):
            print(f"Expires: {d.get('expiration_date_time')}")
        if d.get("display_name"):
            print(f"Display Name: {d.get('display_name')}")

        # iterate results
        async for actions_page in paged_actions:
            print(
                f"Completed: {actions_page.completed}, "
                f"In Progress: {actions_page.in_progress}, "
                f"Failed: {actions_page.failed}, "
                f"Total: {actions_page.total}"
            )

            for action_result in actions_page.task_results or []:
                print(f"\nAction Name: {action_result.name}")
                print(f"Action Status: {action_result.status}")
                print(f"Kind: {action_result.kind}")

                if isinstance(action_result, ConversationPiiOperationResult):
                    for conversation in action_result.results.conversations or []:
                        print(f"Conversation: #{conversation.id}")
                        print("Detected Entities:")

                        for item in conversation.conversation_items or []:
                            for entity in item.entities or []:
                                print(f"  Category: {entity.category}")
                                print(f"  Subcategory: {entity.subcategory}")
                                print(f"  Text: {entity.text}")
                                print(f"  Offset: {entity.offset}")
                                print(f"  Length: {entity.length}")
                                print(f"  Confidence score: {entity.confidence_score}\n")
                                entities_detected.append(entity)

                        if conversation.warnings:
                            print("Warnings:")
                            for warning in conversation.warnings:
                                if isinstance(warning, InputWarning):
                                    print(f"  Code: {warning.code}")
                                    print(f"  Message: {warning.message}")
                        print()
                else:
                    print("  [No supported results to display for this action type]")

        # errors
        if d.get("errors"):
            print("\nErrors:")
            for err in d["errors"]:
                print(f"  Code: {err.code} - {err.message}")


# [END conversation_pii_async]


async def main():
    await sample_conversation_pii_async()


if __name__ == "__main__":
    loop = asyncio.get_event_loop()
    loop.run_until_complete(main())



# coding=utf-8
# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------

"""
FILE: sample_conversation_multi_turn_prediction_async.py

DESCRIPTION:
    Run a multi-turn conversation prediction asynchronously using the
    Conversational AI task. Prints intents and entities, including spans,
    datetime resolutions, and subtype/tag metadata.

USAGE:
    python sample_conversation_multi_turn_prediction_async.py

REQUIRED ENV VARS (for AAD / DefaultAzureCredential):
    AZURE_CONVERSATIONS_ENDPOINT
    AZURE_CLIENT_ID
    AZURE_TENANT_ID
    AZURE_CLIENT_SECRET
    AZURE_CONVERSATIONS_PROJECT_NAME
    AZURE_CONVERSATIONS_DEPLOYMENT_NAME
    
NOTE:
    If you prefer `AzureKeyCredential`, set:
    AZURE_CONVERSATIONS_ENDPOINT
    AZURE_CONVERSATIONS_KEY
"""

# [START conversation_multi_turn_prediction_async]
import os
import asyncio

from azure.identity.aio import DefaultAzureCredential
from azure.ai.language.conversations.aio import ConversationAnalysisClient
from azure.ai.language.conversations.models import (
    # Request
    ConversationalAITask,
    ConversationalAIAnalysisInput,
    ConversationalAIActionContent,
    TextConversation,
    TextConversationItem,
    StringIndexType,
    # Response/result discriminators
    ConversationalAITaskResult,
    ConversationalAIResult,
    ConversationalAIAnalysis,
    ConversationalAIIntent,
    ConversationalAIEntity,
    ConversationItemRange,
    DateTimeResolution,
    EntitySubtype,
    EntityTag,
)


async def sample_conversation_multi_turn_prediction_async():
    # get settings
    endpoint = os.environ["AZURE_CONVERSATIONS_ENDPOINT"]
    project_name = os.environ["AZURE_CONVERSATIONS_PROJECT_NAME"]
    deployment_name = os.environ["AZURE_CONVERSATIONS_DEPLOYMENT_NAME"]

    # AAD credential
    credential = DefaultAzureCredential()

    async with ConversationAnalysisClient(endpoint, credential=credential) as client:
        # Build a small multi-turn dialog
        data = ConversationalAITask(
            analysis_input=ConversationalAIAnalysisInput(
                conversations=[
                    TextConversation(
                        id="order",
                        language="en-GB",
                        conversation_items=[
                            TextConversationItem(id="1", participant_id="user", text="Hi"),
                            TextConversationItem(id="2", participant_id="bot", text="Hello, how can I help you?"),
                            TextConversationItem(
                                id="3",
                                participant_id="user",
                                text="Send an email to Carol about tomorrow's demo",
                            ),
                        ],
                    )
                ]
            ),
            parameters=ConversationalAIActionContent(
                project_name=project_name,
                deployment_name=deployment_name,
                string_index_type=StringIndexType.UTF16_CODE_UNIT,
            ),
        )

        # Async call
        response = await client.analyze_conversation(data)

        if isinstance(response, ConversationalAITaskResult):
            ai_result: ConversationalAIResult = response.result
            if not ai_result or not ai_result.conversations:
                print("No conversations found in result.")
                return

            for conversation in ai_result.conversations or []:
                print(f"Conversation ID: {conversation.id}\n")

                # Intents
                print("Intents:")
                for intent in conversation.intents or []:
                    print(f"  Name: {intent.name}")
                    print(f"  Type: {intent.type}")

                    print("  Conversation Item Ranges:")
                    for rng in intent.conversation_item_ranges or []:
                        print(f"    - Offset: {rng.offset}, Count: {rng.count}")

                    print("\n  Entities (Scoped to Intent):")
                    for ent in intent.entities or []:
                        print(f"    Name: {ent.name}")
                        print(f"    Text: {ent.text}")
                        print(f"    Confidence: {ent.confidence_score}")
                        print(f"    Offset: {ent.offset}, Length: {ent.length}")
                        print(
                            f"    Conversation Item ID: {ent.conversation_item_id}, "
                            f"Index: {ent.conversation_item_index}"
                        )

                        # Date/time resolutions
                        for res in ent.resolutions or []:
                            if isinstance(res, DateTimeResolution):
                                print(
                                    f"    - [DateTimeResolution] SubKind: {res.date_time_sub_kind}, "
                                    f"Timex: {res.timex}, Value: {res.value}"
                                )

                        # Extra information (entity subtype + tags)
                        for extra in ent.extra_information or []:
                            if isinstance(extra, EntitySubtype):
                                print(f"    - [EntitySubtype] Value: {extra.value}")
                                for tag in extra.tags or []:
                                    print(f"      • Tag: {tag.name}, Confidence: {tag.confidence_score}")
                print()

                # Global entities
                print("Global Entities:")
                for ent in conversation.entities or []:
                    print(f"  Name: {ent.name}")
                    print(f"  Text: {ent.text}")
                    print(f"  Confidence: {ent.confidence_score}")
                    print(f"  Offset: {ent.offset}, Length: {ent.length}")
                    print(
                        f"  Conversation Item ID: {ent.conversation_item_id}, " f"Index: {ent.conversation_item_index}"
                    )

                    for extra in ent.extra_information or []:
                        if isinstance(extra, EntitySubtype):
                            print(f"    - [EntitySubtype] Value: {extra.value}")
                            for tag in extra.tags or []:
                                print(f"      • Tag: {tag.name}, Confidence: {tag.confidence_score}")
                print("-" * 40)
        else:
            print("No Conversational AI result returned.")


# [END conversation_multi_turn_prediction_async]


async def main():
    await sample_conversation_multi_turn_prediction_async()


if __name__ == "__main__":
    loop = asyncio.get_event_loop()
    loop.run_until_complete(main())



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------
"""Customize generated code here.

Follow our quickstart for examples: https://aka.ms/azsdk/python/dpcodegen/python/customize
"""
from ._models import (
    AnalyzeConversationOperationInput,
    MultiLanguageConversationInput,
    SummarizationOperationAction,
    ConversationSummarizationActionContent,
    AnalyzeConversationOperationAction,
    ConversationPiiActionContent,
    PiiOperationAction,
    CharacterMaskPolicyType,
    EntityMaskTypePolicyType,
    NoMaskPolicyType,
)
from ._enums import RedactionCharacter


def patch_sdk():
    """Do not remove from this file.

    `patch_sdk` is a last resort escape hatch that allows you to do customizations
    you can't accomplish using the techniques described in
    https://aka.ms/azsdk/python/dpcodegen/python/customize
    """


__all__ = [
    "AnalyzeConversationOperationInput",
    "MultiLanguageConversationInput",
    "SummarizationOperationAction",
    "ConversationSummarizationActionContent",
    "AnalyzeConversationOperationAction",
    "ConversationPiiActionContent",
    "PiiOperationAction",
    "CharacterMaskPolicyType",
    "RedactionCharacter",
    "EntityMaskTypePolicyType",
    "NoMaskPolicyType",
]



# coding=utf-8
# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------

"""
FILE: sample_conversation_summarization.py

DESCRIPTION:
    This sample demonstrates how to summarize a conversation using CLU's conversation
    summarization capability (sync).

USAGE:
    python sample_conversation_summarization.py

REQUIRED ENV VARS (for AAD / DefaultAzureCredential):
    AZURE_CONVERSATIONS_ENDPOINT
    AZURE_CLIENT_ID
    AZURE_TENANT_ID
    AZURE_CLIENT_SECRET

NOTE:
    If you want to use AzureKeyCredential instead, set:
      - AZURE_CONVERSATIONS_ENDPOINT
      - AZURE_CONVERSATIONS_KEY
"""

# [START conversation_summarization]
import os

from azure.identity import DefaultAzureCredential
from azure.ai.language.conversations import ConversationAnalysisClient
from azure.ai.language.conversations.models import (
    TextConversationItem,
    TextConversation,
    ParticipantRole,
    MultiLanguageConversationInput,
    SummarizationOperationAction,
    ConversationSummarizationActionContent,
    SummaryAspect,
    AnalyzeConversationOperationInput,
    SummarizationOperationResult,
    ConversationError,
)


def sample_conversation_summarization():
    # settings
    endpoint = os.environ["AZURE_CONVERSATIONS_ENDPOINT"]
    credential = DefaultAzureCredential()

    # Build conversation input
    conversation_items = [
        TextConversationItem(
            id="1", participant_id="Agent_1", text="Hello, how can I help you?", role=ParticipantRole.AGENT
        ),
        TextConversationItem(
            id="2",
            participant_id="Customer_1",
            text="How to upgrade Office? I am getting error messages the whole day.",
            role=ParticipantRole.CUSTOMER,
        ),
        TextConversationItem(
            id="3",
            participant_id="Agent_1",
            text="Press the upgrade button please. Then sign in and follow the instructions.",
            role=ParticipantRole.AGENT,
        ),
    ]

    conversation_input = MultiLanguageConversationInput(
        conversations=[TextConversation(id="1", language="en", conversation_items=conversation_items)]
    )

    # Build the operation input and inline actions
    operation_input = AnalyzeConversationOperationInput(
        conversation_input=conversation_input,
        actions=[
            SummarizationOperationAction(
                name="Issue task",
                action_content=ConversationSummarizationActionContent(summary_aspects=[SummaryAspect.ISSUE]),
            ),
            SummarizationOperationAction(
                name="Resolution task",
                action_content=ConversationSummarizationActionContent(summary_aspects=[SummaryAspect.RESOLUTION]),
            ),
        ],
    )

    client = ConversationAnalysisClient(endpoint, credential=credential)

    poller = client.begin_analyze_conversation_job(body=operation_input)

    # Operation ID
    op_id = poller.details.get("operation_id")
    if op_id:
        print(f"Operation ID: {op_id}")

    # Wait for result
    paged_actions = poller.result()

    # Final-state metadata
    d = poller.details
    print(f"Job ID: {d.get('job_id')}")
    print(f"Status: {d.get('status')}")
    print(f"Created: {d.get('created_date_time')}")
    print(f"Last Updated: {d.get('last_updated_date_time')}")
    if d.get("expiration_date_time"):
        print(f"Expires: {d.get('expiration_date_time')}")
    if d.get("display_name"):
        print(f"Display Name: {d.get('display_name')}")

    # Iterate results
    for actions_page in paged_actions:
        print(
            f"Completed: {actions_page.completed}, "
            f"In Progress: {actions_page.in_progress}, "
            f"Failed: {actions_page.failed}, "
            f"Total: {actions_page.total}"
        )

        for action_result in actions_page.task_results or []:
            if isinstance(action_result, SummarizationOperationResult):
                for conversation in action_result.results.conversations or []:
                    print(f"  Conversation ID: {conversation.id}")
                    print("  Summaries:")
                    for summary in conversation.summaries or []:
                        print(f"    Aspect: {summary.aspect}")
                        print(f"    Text: {summary.text}")
                    if conversation.warnings:
                        print("  Warnings:")
                        for warning in conversation.warnings:
                            print(f"    Code: {warning.code}, Message: {warning.message}")
            else:
                print("  [No supported results to display for this action type]")

    # Errors
    if d.get("errors"):
        print("\nErrors:")
        for error in d["errors"]:
            if isinstance(error, ConversationError):
                print(f"  Code: {error.code} - {error.message}")


# [END conversation_summarization]


def main():
    sample_conversation_summarization()


if __name__ == "__main__":
    main()



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
# pylint: disable=wrong-import-position

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from ._patch import *  # pylint: disable=unused-wildcard-import


from ._models import (  # type: ignore
    AgeResolution,
    AnalysisConfig,
    AnalyzeConversationActionResult,
    AnalyzeConversationInput,
    AnalyzeConversationOperationResult,
    AnalyzeConversationOperationState,
    AnalyzeConversationResult,
    AnswerSpan,
    AnswersResult,
    AreaResolution,
    AudioTiming,
    BooleanResolution,
    ConversationActionContent,
    ConversationActionResult,
    ConversationActions,
    ConversationAnalysisInput,
    ConversationCallingConfig,
    ConversationConfig,
    ConversationEntity,
    ConversationEntityExtraInformation,
    ConversationError,
    ConversationInput,
    ConversationIntent,
    ConversationItemRange,
    ConversationLanguageUnderstandingInput,
    ConversationPiiItemResult,
    ConversationPiiOperationResult,
    ConversationPiiResults,
    ConversationPrediction,
    ConversationRequestStatistics,
    ConversationResult,
    ConversationStatistics,
    ConversationTargetIntentResult,
    ConversationalAIActionContent,
    ConversationalAIAnalysis,
    ConversationalAIAnalysisInput,
    ConversationalAIEntity,
    ConversationalAIIntent,
    ConversationalAIResult,
    ConversationalAITask,
    ConversationalAITaskResult,
    ConversationalPiiResult,
    ConversationsSummaryResult,
    CurrencyResolution,
    CustomSummarizationOperationResult,
    CustomSummaryResult,
    DateTimeResolution,
    DocumentError,
    EntitySubtype,
    EntityTag,
    ErrorResponse,
    InformationResolution,
    InnerErrorModel,
    InputWarning,
    ItemizedSummaryContext,
    KnowledgeBaseAnswer,
    KnowledgeBaseAnswerContext,
    KnowledgeBaseAnswerDialog,
    KnowledgeBaseAnswerPrompt,
    LengthResolution,
    ListKey,
    LuisCallingConfig,
    LuisConfig,
    LuisResult,
    LuisTargetIntentResult,
    MetadataFilter,
    MetadataRecord,
    NamedEntity,
    NonLinkedTargetIntentResult,
    NumberResolution,
    NumericRangeResolution,
    OrchestrationPrediction,
    OrdinalResolution,
    PredictionBase,
    QueryFilters,
    QuestionAnsweringConfig,
    QuestionAnsweringTargetIntentResult,
    QuestionAnswersConfig,
    RedactedTranscriptContent,
    RegexKey,
    RequestStatistics,
    ResolutionBase,
    ShortAnswerConfig,
    SpeedResolution,
    SummarizationOperationResult,
    SummaryResult,
    SummaryResultItem,
    TargetIntentResult,
    TemperatureResolution,
    TemporalSpanResolution,
    TextConversation,
    TextConversationItem,
    VolumeResolution,
    WeightResolution,
)

from ._enums import (  # type: ignore
    AgeUnit,
    AnalyzeConversationInputKind,
    AnalyzeConversationOperationResultsKind,
    AnalyzeConversationResultKind,
    AreaUnit,
    ConversationActionState,
    ConversationDomain,
    ConversationErrorCode,
    DateTimeSubKind,
    ExtraInformationKind,
    InformationUnit,
    InnerErrorCode,
    InputModality,
    LengthUnit,
    LogicalOperationKind,
    NumberKind,
    ParticipantRole,
    ProjectKind,
    RangeKind,
    RankerKind,
    RelativeTo,
    ResolutionKind,
    SpeedUnit,
    StringIndexType,
    SummaryAspect,
    SummaryLengthBucket,
    TargetProjectKind,
    TemperatureUnit,
    TemporalModifier,
    VolumeUnit,
    WeightUnit,
)
from ._patch import __all__ as _patch_all
from ._patch import *
from ._patch import patch_sdk as _patch_sdk

__all__ = [
    "AgeResolution",
    "AnalysisConfig",
    "AnalyzeConversationActionResult",
    "AnalyzeConversationInput",
    "AnalyzeConversationOperationResult",
    "AnalyzeConversationOperationState",
    "AnalyzeConversationResult",
    "AnswerSpan",
    "AnswersResult",
    "AreaResolution",
    "AudioTiming",
    "BooleanResolution",
    "ConversationActionContent",
    "ConversationActionResult",
    "ConversationActions",
    "ConversationAnalysisInput",
    "ConversationCallingConfig",
    "ConversationConfig",
    "ConversationEntity",
    "ConversationEntityExtraInformation",
    "ConversationError",
    "ConversationInput",
    "ConversationIntent",
    "ConversationItemRange",
    "ConversationLanguageUnderstandingInput",
    "ConversationPiiItemResult",
    "ConversationPiiOperationResult",
    "ConversationPiiResults",
    "ConversationPrediction",
    "ConversationRequestStatistics",
    "ConversationResult",
    "ConversationStatistics",
    "ConversationTargetIntentResult",
    "ConversationalAIActionContent",
    "ConversationalAIAnalysis",
    "ConversationalAIAnalysisInput",
    "ConversationalAIEntity",
    "ConversationalAIIntent",
    "ConversationalAIResult",
    "ConversationalAITask",
    "ConversationalAITaskResult",
    "ConversationalPiiResult",
    "ConversationsSummaryResult",
    "CurrencyResolution",
    "CustomSummarizationOperationResult",
    "CustomSummaryResult",
    "DateTimeResolution",
    "DocumentError",
    "EntitySubtype",
    "EntityTag",
    "ErrorResponse",
    "InformationResolution",
    "InnerErrorModel",
    "InputWarning",
    "ItemizedSummaryContext",
    "KnowledgeBaseAnswer",
    "KnowledgeBaseAnswerContext",
    "KnowledgeBaseAnswerDialog",
    "KnowledgeBaseAnswerPrompt",
    "LengthResolution",
    "ListKey",
    "LuisCallingConfig",
    "LuisConfig",
    "LuisResult",
    "LuisTargetIntentResult",
    "MetadataFilter",
    "MetadataRecord",
    "NamedEntity",
    "NonLinkedTargetIntentResult",
    "NumberResolution",
    "NumericRangeResolution",
    "OrchestrationPrediction",
    "OrdinalResolution",
    "PredictionBase",
    "QueryFilters",
    "QuestionAnsweringConfig",
    "QuestionAnsweringTargetIntentResult",
    "QuestionAnswersConfig",
    "RedactedTranscriptContent",
    "RegexKey",
    "RequestStatistics",
    "ResolutionBase",
    "ShortAnswerConfig",
    "SpeedResolution",
    "SummarizationOperationResult",
    "SummaryResult",
    "SummaryResultItem",
    "TargetIntentResult",
    "TemperatureResolution",
    "TemporalSpanResolution",
    "TextConversation",
    "TextConversationItem",
    "VolumeResolution",
    "WeightResolution",
    "AgeUnit",
    "AnalyzeConversationInputKind",
    "AnalyzeConversationOperationResultsKind",
    "AnalyzeConversationResultKind",
    "AreaUnit",
    "ConversationActionState",
    "ConversationDomain",
    "ConversationErrorCode",
    "DateTimeSubKind",
    "ExtraInformationKind",
    "InformationUnit",
    "InnerErrorCode",
    "InputModality",
    "LengthUnit",
    "LogicalOperationKind",
    "NumberKind",
    "ParticipantRole",
    "ProjectKind",
    "RangeKind",
    "RankerKind",
    "RelativeTo",
    "ResolutionKind",
    "SpeedUnit",
    "StringIndexType",
    "SummaryAspect",
    "SummaryLengthBucket",
    "TargetProjectKind",
    "TemperatureUnit",
    "TemporalModifier",
    "VolumeUnit",
    "WeightUnit",
]
__all__.extend([p for p in _patch_all if p not in __all__])  # pyright: ignore
_patch_sdk()



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from enum import Enum
from azure.core import CaseInsensitiveEnumMeta


class AgeUnit(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The Age Unit of measurement."""

    UNSPECIFIED = "Unspecified"
    """Unspecified age unit"""
    YEAR = "Year"
    """Year age unit"""
    MONTH = "Month"
    """Month age unit"""
    WEEK = "Week"
    """Week age unit"""
    DAY = "Day"
    """Day age unit"""


class AnalyzeConversationInputKind(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Enumeration of supported Conversation tasks."""

    CONVERSATION = "Conversation"
    """Conversation task kind"""
    CONVERSATIONAL_AI = "ConversationalAI"
    """Conversation task kind"""


class AnalyzeConversationOperationActionKind(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Enumeration of supported analysis tasks on a collection of conversations."""

    CONVERSATIONAL_SUMMARIZATION_TASK = "ConversationalSummarizationTask"
    """Conversational Summarization Task"""
    CONVERSATIONAL_PII_TASK = "ConversationalPIITask"
    """Conversational PII Task"""
    CUSTOM_CONVERSATIONAL_SUMMARIZATION_TASK = "CustomConversationalSummarizationTask"
    """Custom Conversational Summarization Task"""


class AnalyzeConversationOperationResultsKind(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Enumeration of results for supported conversation analysis Results."""

    SUMMARIZATION_OPERATION_RESULTS = "conversationalSummarizationResults"
    """Conversational Summarization Results"""
    PII_OPERATION_RESULTS = "conversationalPIIResults"
    """Conversational PII Results"""
    CUSTOM_SUMMARIZATION_OPERATION_RESULTS = "customConversationalSummarizationResults"
    """Custom Conversational Summarization Results"""


class AnalyzeConversationResultKind(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Enumeration of supported conversational task results."""

    CONVERSATION_RESULT = "ConversationResult"
    """Conversation result task kind"""
    CONVERSATIONAL_AI_RESULT = "ConversationalAIResult"
    """Conversation result task kind"""


class AreaUnit(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The area unit of measurement."""

    UNSPECIFIED = "Unspecified"
    """Unspecified area unit"""
    SQUARE_KILOMETER = "SquareKilometer"
    """Square kilometer area unit"""
    SQUARE_HECTOMETER = "SquareHectometer"
    """Square hectometer area unit"""
    SQUARE_DECAMETER = "SquareDecameter"
    """Square decameter area unit"""
    SQUARE_DECIMETER = "SquareDecimeter"
    """Square decimeter area unit"""
    SQUARE_METER = "SquareMeter"
    """Square meter area unit"""
    SQUARE_CENTIMETER = "SquareCentimeter"
    """Square centimeter area unit"""
    SQUARE_MILLIMETER = "SquareMillimeter"
    """Square millimeter area unit"""
    SQUARE_INCH = "SquareInch"
    """Square inch area unit"""
    SQUARE_FOOT = "SquareFoot"
    """Square foot area unit"""
    SQUARE_MILE = "SquareMile"
    """Square mile area unit"""
    SQUARE_YARD = "SquareYard"
    """Square yard area unit"""
    ACRE = "Acre"
    """Acre area unit"""


class ConversationActionState(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The status of the task at the mentioned last update time."""

    NOT_STARTED = "notStarted"
    """Not started state"""
    RUNNING = "running"
    """Running state"""
    SUCCEEDED = "succeeded"
    """Succeeded state"""
    PARTIALLY_COMPLETED = "partiallyCompleted"
    """Partially completed state"""
    FAILED = "failed"
    """Failed state"""
    CANCELLED = "cancelled"
    """Cancelled state"""
    CANCELLING = "cancelling"
    """Cancelling state"""


class ConversationDomain(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Enumeration of supported conversational domains."""

    FINANCE = "finance"
    """Enumeration of supported conversational domains."""
    HEALTHCARE = "healthcare"
    """Enumeration of supported conversational domains."""
    GENERIC = "generic"
    """Enumeration of supported conversational domains."""


class ConversationErrorCode(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Human-readable error code."""

    INVALID_REQUEST = "InvalidRequest"
    """Invalid request error"""
    INVALID_ARGUMENT = "InvalidArgument"
    """Invalid argument error"""
    UNAUTHORIZED = "Unauthorized"
    """Unauthorized access error"""
    FORBIDDEN = "Forbidden"
    """Forbidden access error"""
    NOT_FOUND = "NotFound"
    """Not found error"""
    PROJECT_NOT_FOUND = "ProjectNotFound"
    """Project not found error"""
    OPERATION_NOT_FOUND = "OperationNotFound"
    """Operation not found error"""
    AZURE_COGNITIVE_SEARCH_NOT_FOUND = "AzureCognitiveSearchNotFound"
    """Azure Cognitive Search not found error"""
    AZURE_COGNITIVE_SEARCH_INDEX_NOT_FOUND = "AzureCognitiveSearchIndexNotFound"
    """Azure Cognitive Search index not found error"""
    TOO_MANY_REQUESTS = "TooManyRequests"
    """Too many requests error"""
    AZURE_COGNITIVE_SEARCH_THROTTLING = "AzureCognitiveSearchThrottling"
    """Azure Cognitive Search throttling error"""
    AZURE_COGNITIVE_SEARCH_INDEX_LIMIT_REACHED = "AzureCognitiveSearchIndexLimitReached"
    """Azure Cognitive Search index limit reached error"""
    INTERNAL_SERVER_ERROR = "InternalServerError"
    """Internal server error"""
    SERVICE_UNAVAILABLE = "ServiceUnavailable"
    """Service unavailable error"""
    TIMEOUT = "Timeout"
    """Timeout error"""
    QUOTA_EXCEEDED = "QuotaExceeded"
    """Quota exceeded error"""
    CONFLICT = "Conflict"
    """Conflict error"""
    WARNING = "Warning"
    """Warning error"""


class ConversationPiiCategories(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Describes the PII categories to return for detection. If not provided, 'default' categories
    will be returned which will vary with the language.
    """

    ADDRESS = "Address"
    """Address category"""
    CREDIT_CARD = "CreditCard"
    """Credit card category"""
    EMAIL = "Email"
    """Email category"""
    PERSON = "Person"
    """Person category"""
    NUMERIC_IDENTIFIER = "NumericIdentifier"
    """Numeric identifier category"""
    PHONE = "Phone"
    """Phone category"""
    US_SOCIAL_SECURITY_NUMBER = "USSocialSecurityNumber"
    """US social security number category"""
    DRIVERS_LICENSE_NUMBER = "DriversLicenseNumber"
    """Driver's license number category"""
    PASSPORT_NUMBER = "PassportNumber"
    """Passport number category"""
    PERSON_TYPE = "PersonType"
    """Person type category"""
    ORGANIZATION = "Organization"
    """Organization category"""
    ABA_ROUTING_NUMBER = "ABARoutingNumber"
    """ABA routing number category"""
    BANK_ACCOUNT_NUMBER = "BankAccountNumber"
    """Bank account number category"""
    DATE_OF_BIRTH = "DateOfBirth"
    """Date of birth category"""
    INTERNATIONAL_BANKING_ACCOUNT_NUMBER = "InternationalBankingAccountNumber"
    """International Bank Account Number category"""
    SWIFT_CODE = "SWIFTCode"
    """SWIFT code category"""
    VEHICLE_IDENTIFICATION_NUMBER = "VehicleIdentificationNumber"
    """Vehicle identification number category"""
    AGE = "Age"
    """Age category"""
    DATE = "Date"
    """Date category"""
    ZIP_CODE = "ZipCode"
    """Zip code category"""
    GOVERNMENT_ISSUED_ID = "GovernmentIssuedId"
    """Government issued ID category"""
    CVV = "CVV"
    """Card verification value category"""
    HEALTH_CARD_NUMBER = "HealthCardNumber"
    """Health card number category"""
    CA_SOCIAL_INSURANCE_NUMBER = "CASocialInsuranceNumber"
    """CA Social insurance number category"""
    US_MEDICARE_BENEFICIARY_ID = "USMedicareBeneficiaryId"
    """US Medicare beneficiary ID category"""
    GITHUB_ACCOUNT = "GithubAccount"
    """Github account category"""
    LOCATION = "Location"
    """Location category"""
    GPE = "GPE"
    """GPE category"""
    ALL = "All"
    """All categories"""
    DEFAULT = "Default"
    """Default categories"""


class ConversationPiiCategoryExclusions(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Describes the PII categories to exclude for detection. If not provided, 'default' categories
    will be returned which will vary with the language.
    """

    ADDRESS = "Address"
    """Address category"""
    CREDIT_CARD = "CreditCard"
    """Credit card category"""
    EMAIL = "Email"
    """Email category"""
    PERSON = "Person"
    """Person category"""
    NUMERIC_IDENTIFIER = "NumericIdentifier"
    """Numeric identifier category"""
    PHONE = "Phone"
    """Phone category"""
    US_SOCIAL_SECURITY_NUMBER = "USSocialSecurityNumber"
    """US social security number category"""
    DRIVERS_LICENSE_NUMBER = "DriversLicenseNumber"
    """Driver's license number category"""
    PASSPORT_NUMBER = "PassportNumber"
    """Passport number category"""
    PERSON_TYPE = "PersonType"
    """Person type category"""
    ORGANIZATION = "Organization"
    """Organization category"""
    ABA_ROUTING_NUMBER = "ABARoutingNumber"
    """ABA routing number category"""
    BANK_ACCOUNT_NUMBER = "BankAccountNumber"
    """Bank account number category"""
    DATE_OF_BIRTH = "DateOfBirth"
    """Date of birth category"""
    INTERNATIONAL_BANKING_ACCOUNT_NUMBER = "InternationalBankingAccountNumber"
    """International Bank Account Number category"""
    SWIFT_CODE = "SWIFTCode"
    """SWIFT code category"""
    VEHICLE_IDENTIFICATION_NUMBER = "VehicleIdentificationNumber"
    """Vehicle identification number category"""
    AGE = "Age"
    """Age category"""
    DATE = "Date"
    """Date category"""
    ZIP_CODE = "ZipCode"
    """Zip code category"""
    GOVERNMENT_ISSUED_ID = "GovernmentIssuedId"
    """Government issued ID category"""
    CVV = "CVV"
    """Card verification value category"""
    HEALTH_CARD_NUMBER = "HealthCardNumber"
    """Health card number category"""
    CA_SOCIAL_INSURANCE_NUMBER = "CASocialInsuranceNumber"
    """CA Social insurance number category"""
    US_MEDICARE_BENEFICIARY_ID = "USMedicareBeneficiaryId"
    """US Medicare beneficiary ID category"""
    GITHUB_ACCOUNT = "GithubAccount"
    """Github account category"""
    LOCATION = "Location"
    """Location category"""
    GPE = "GPE"
    """GPE category"""


class DateTimeSubKind(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The Datetime SubKind."""

    TIME = "Time"
    """Time subkind"""
    DATE = "Date"
    """Date subkind"""
    DATE_TIME = "DateTime"
    """DateTime subkind"""
    DURATION = "Duration"
    """Duration subkind"""
    SET = "Set"
    """Set subkind"""


class ExtraInformationKind(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The extra information object kind."""

    ENTITY_SUBTYPE = "EntitySubtype"
    """Entity subtype extra information kind"""
    LIST_KEY = "ListKey"
    """List key extra information kind"""
    REGEX_KEY = "RegexKey"
    """Regex key extra information kind"""


class InformationUnit(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The information (data) Unit of measurement."""

    UNSPECIFIED = "Unspecified"
    """Unspecified information unit"""
    BIT = "Bit"
    """Bit information unit"""
    KILOBIT = "Kilobit"
    """Kilobit information unit"""
    MEGABIT = "Megabit"
    """Megabit information unit"""
    GIGABIT = "Gigabit"
    """Gigabit information unit"""
    TERABIT = "Terabit"
    """Terabit information unit"""
    PETABIT = "Petabit"
    """Petabit information unit"""
    BYTE = "Byte"
    """Byte information unit"""
    KILOBYTE = "Kilobyte"
    """Kilobyte information unit"""
    MEGABYTE = "Megabyte"
    """Megabyte information unit"""
    GIGABYTE = "Gigabyte"
    """Gigabyte information unit"""
    TERABYTE = "Terabyte"
    """Terabyte information unit"""
    PETABYTE = "Petabyte"
    """Petabyte information unit"""


class InnerErrorCode(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Human-readable error code."""

    INVALID_REQUEST = "InvalidRequest"
    """Invalid request error"""
    INVALID_PARAMETER_VALUE = "InvalidParameterValue"
    """Invalid parameter value error"""
    KNOWLEDGE_BASE_NOT_FOUND = "KnowledgeBaseNotFound"
    """Knowledge base not found error"""
    AZURE_COGNITIVE_SEARCH_NOT_FOUND = "AzureCognitiveSearchNotFound"
    """Azure Cognitive Search not found error"""
    AZURE_COGNITIVE_SEARCH_THROTTLING = "AzureCognitiveSearchThrottling"
    """Azure Cognitive Search throttling error"""
    EXTRACTION_FAILURE = "ExtractionFailure"
    """Extraction failure error"""
    INVALID_REQUEST_BODY_FORMAT = "InvalidRequestBodyFormat"
    """Invalid request body format error"""
    EMPTY_REQUEST = "EmptyRequest"
    """Empty request error"""
    MISSING_INPUT_DOCUMENTS = "MissingInputDocuments"
    """Missing input documents error"""
    INVALID_DOCUMENT = "InvalidDocument"
    """Invalid document error"""
    MODEL_VERSION_INCORRECT = "ModelVersionIncorrect"
    """Model version incorrect error"""
    INVALID_DOCUMENT_BATCH = "InvalidDocumentBatch"
    """Invalid document batch error"""
    UNSUPPORTED_LANGUAGE_CODE = "UnsupportedLanguageCode"
    """Unsupported language code error"""
    INVALID_COUNTRY_HINT = "InvalidCountryHint"
    """Invalid country hint error"""


class InputModality(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Enumeration of supported conversational modalities."""

    TRANSCRIPT = "transcript"
    """Transcript input modality"""
    TEXT = "text"
    """Text input modality"""


class LengthUnit(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The length unit of measurement."""

    UNSPECIFIED = "Unspecified"
    """Unspecified length unit"""
    KILOMETER = "Kilometer"
    """Kilometer length unit"""
    HECTOMETER = "Hectometer"
    """Hectometer length unit"""
    DECAMETER = "Decameter"
    """Decameter length unit"""
    METER = "Meter"
    """Meter length unit"""
    DECIMETER = "Decimeter"
    """Decimeter length unit"""
    CENTIMETER = "Centimeter"
    """Centimeter length unit"""
    MILLIMETER = "Millimeter"
    """Millimeter length unit"""
    MICROMETER = "Micrometer"
    """Micrometer length unit"""
    NANOMETER = "Nanometer"
    """Nanometer length unit"""
    PICOMETER = "Picometer"
    """Picometer length unit"""
    MILE = "Mile"
    """Mile length unit"""
    YARD = "Yard"
    """Yard length unit"""
    INCH = "Inch"
    """Inch length unit"""
    FOOT = "Foot"
    """Foot length unit"""
    LIGHT_YEAR = "LightYear"
    """Light year length unit"""
    POINT = "Pt"
    """Point length unit"""


class LogicalOperationKind(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Set to 'OR' or 'AND' for using corresponding logical operation."""

    AND = "AND"
    """Logical AND"""
    OR = "OR"
    """Logical OR"""


class NumberKind(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The type of the extracted number entity."""

    INTEGER = "Integer"
    """Integer number"""
    DECIMAL = "Decimal"
    """Decimal number"""
    POWER = "Power"
    """Power number"""
    FRACTION = "Fraction"
    """Fraction number"""
    PERCENT = "Percent"
    """Percent number"""
    UNSPECIFIED = "Unspecified"
    """Unspecified number kind"""


class ParticipantRole(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Role of the participant."""

    CUSTOMER = "customer"
    """The participant is a customer."""
    AGENT = "agent"
    """The participant is an agent."""
    GENERIC = "generic"
    """The is a generic participant."""


class ProjectKind(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The type of the project."""

    CONVERSATION = "Conversation"
    """Conversation type"""
    ORCHESTRATION = "Orchestration"
    """Orchestration type"""
    CONVERSATIONAL_AI = "ConversationalAI"
    """Conversation type"""


class RangeKind(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The kind of range that the resolution object represents."""

    NUMBER = "Number"
    """Number range"""
    SPEED = "Speed"
    """Speed range"""
    WEIGHT = "Weight"
    """Weight range"""
    LENGTH = "Length"
    """Length range"""
    VOLUME = "Volume"
    """Volume range"""
    AREA = "Area"
    """Area range"""
    AGE = "Age"
    """Age range"""
    INFORMATION = "Information"
    """Information range"""
    TEMPERATURE = "Temperature"
    """Temperature range"""
    CURRENCY = "Currency"
    """Currency range"""


class RankerKind(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Type of ranker to be used."""

    DEFAULT = "Default"
    """Default ranker."""
    QUESTION_ONLY = "QuestionOnly"
    """Question only ranker."""


class RedactionCharacter(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Optional parameter to use a Custom Character to be used for redaction in PII responses. Default
    character will be * as before. We allow specific ascii characters for redaction.
    """

    EXCLAMATION_POINT = "!"
    """Exclamation point character"""
    NUMBER_SIGN = "#"
    """Number sign character"""
    DOLLAR = "$"
    """Dollar sign character"""
    PERCENT = "%"
    """Percent sign character"""
    AMPERSAND = "&"
    """Ampersand character"""
    ASTERISK = "*"
    """Asterisk character"""
    PLUS = "+"
    """Plus sign character"""
    MINUS = "-"
    """Minus sign character"""
    EQUALS = "="
    """Equals sign character"""
    QUESTION_MARK = "?"
    """Question mark character"""
    AT_SIGN = "@"
    """At sign character"""
    CARET = "^"
    """Caret character"""
    UNDERSCORE = "_"
    """Underscore character"""
    TILDE = "~"
    """Tilde character"""


class RedactionPolicyKind(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Kinds of redaction policies supported."""

    NO_MASK = "noMask"
    """Do not mask detected entities"""
    CHARACTER_MASK = "characterMask"
    """Mask detected entities with redaction character"""
    ENTITY_MASK = "entityMask"
    """Mask detected entities with entity type"""


class RelativeTo(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The reference point that the ordinal number denotes."""

    CURRENT = "Current"
    """Current state"""
    END = "End"
    """End state"""
    START = "Start"
    """Start state"""


class ResolutionKind(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The entity resolution object kind."""

    BOOLEAN_RESOLUTION = "BooleanResolution"
    """Resolution of a boolean entity"""
    DATE_TIME_RESOLUTION = "DateTimeResolution"
    """Resolution of a date/time entity"""
    NUMBER_RESOLUTION = "NumberResolution"
    """Resolution of a number entity"""
    ORDINAL_RESOLUTION = "OrdinalResolution"
    """Resolution of an ordinal entity"""
    SPEED_RESOLUTION = "SpeedResolution"
    """Resolution of a speed entity"""
    WEIGHT_RESOLUTION = "WeightResolution"
    """Resolution of a weight entity"""
    LENGTH_RESOLUTION = "LengthResolution"
    """Resolution of a length entity"""
    VOLUME_RESOLUTION = "VolumeResolution"
    """Resolution of a volume entity"""
    AREA_RESOLUTION = "AreaResolution"
    """Resolution of an area entity"""
    AGE_RESOLUTION = "AgeResolution"
    """Resolution of an age entity"""
    INFORMATION_RESOLUTION = "InformationResolution"
    """Resolution of an information entity"""
    TEMPERATURE_RESOLUTION = "TemperatureResolution"
    """Resolution of a temperature entity"""
    CURRENCY_RESOLUTION = "CurrencyResolution"
    """Resolution of a currency entity"""
    NUMERIC_RANGE_RESOLUTION = "NumericRangeResolution"
    """Resolution of a numeric range entity"""
    TEMPORAL_SPAN_RESOLUTION = "TemporalSpanResolution"
    """Resolution of a temporal span entity"""


class SpeedUnit(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The speed Unit of measurement."""

    UNSPECIFIED = "Unspecified"
    """Unspecified speed unit"""
    METERS_PER_SECOND = "MetersPerSecond"
    """Meters per second speed unit"""
    KILOMETERS_PER_HOUR = "KilometersPerHour"
    """Kilometers per hour speed unit"""
    KILOMETERS_PER_MINUTE = "KilometersPerMinute"
    """Kilometers per minute speed unit"""
    KILOMETERS_PER_SECOND = "KilometersPerSecond"
    """Kilometers per second speed unit"""
    MILES_PER_HOUR = "MilesPerHour"
    """Miles per hour speed unit"""
    KNOT = "Knot"
    """Knot speed unit"""
    FOOT_PER_SECOND = "FootPerSecond"
    """Foot per second speed unit"""
    FOOT_PER_MINUTE = "FootPerMinute"
    """Foot per minute speed unit"""
    YARDS_PER_MINUTE = "YardsPerMinute"
    """Yards per minute speed unit"""
    YARDS_PER_SECOND = "YardsPerSecond"
    """Yards per second speed unit"""
    METERS_PER_MILLISECOND = "MetersPerMillisecond"
    """Meters per millisecond speed unit"""
    CENTIMETERS_PER_MILLISECOND = "CentimetersPerMillisecond"
    """Centimeters per millisecond speed unit"""
    KILOMETERS_PER_MILLISECOND = "KilometersPerMillisecond"
    """Kilometers per millisecond speed unit"""


class StringIndexType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Specifies the method used to interpret string offsets.  Defaults to Text Elements (Graphemes)
    according to Unicode v8.0.0. For additional information see
    `https://aka.ms/text-analytics-offsets <https://aka.ms/text-analytics-offsets>`_.
    """

    TEXT_ELEMENTS_V8 = "TextElements_v8"
    """Returned offset and length values will correspond to TextElements (Graphemes and Grapheme
    clusters) confirming to the Unicode 8.0.0 standard. Use this option if your application is
    written in .Net Framework or .Net Core and you will be using StringInfo."""
    UNICODE_CODE_POINT = "UnicodeCodePoint"
    """Returned offset and length values will correspond to Unicode code points. Use this option if
    your application is written in a language that support Unicode, for example Python."""
    UTF16_CODE_UNIT = "Utf16CodeUnit"
    """Returned offset and length values will correspond to UTF-16 code units. Use this option if your
    application is written in a language that support Unicode, for example Java, JavaScript."""


class SummaryAspect(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Supported parameters for a conversational summarization task."""

    ISSUE = "issue"
    """Summary of issues in transcripts of web chats and service calls between customer-service agents
    and customers."""
    RESOLUTION = "resolution"
    """Summary of resolutions in transcripts of web chats and service calls between customer-service
    agents and customers."""
    CHAPTER_TITLE = "chapterTitle"
    """Chapter title of any conversation. It's usually one phrase or several phrases naturally
    combined. Long conversations tend to have more chapters. You can find the chapter boundary from
    the summary context."""
    NARRATIVE = "narrative"
    """Generic narrative summary of any conversation. It generally converts the conversational
    language into formal written language, compresses the text length, and keeps the salient
    information."""
    RECAP = "recap"
    """A concise one-paragraph summary to provide a quick overview."""
    FOLLOW_UP_TASKS = "follow-up tasks"
    """Action items and tasks that arose during a meeting."""


class SummaryLengthBucket(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Enum that defines the length of the output summaries."""

    SHORT = "short"
    """Instructs model to generate shorter length summaries."""
    MEDIUM = "medium"
    """Instructs model to generate medium length summaries."""
    LONG = "long"
    """Instructs model to generate longer length summaries."""


class TargetProjectKind(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The type of a target service."""

    LUIS = "Luis"
    """Luis target service type"""
    CONVERSATION = "Conversation"
    """Conversation target service type"""
    QUESTION_ANSWERING = "QuestionAnswering"
    """QuestionAnswering target service type"""
    NON_LINKED = "NonLinked"
    """NonLinked target service type"""


class TemperatureUnit(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The temperature Unit of measurement."""

    UNSPECIFIED = "Unspecified"
    """Unspecified temperature unit"""
    FAHRENHEIT = "Fahrenheit"
    """Fahrenheit temperature unit"""
    KELVIN = "Kelvin"
    """Kelvin temperature unit"""
    RANKINE = "Rankine"
    """Rankine temperature unit"""
    CELSIUS = "Celsius"
    """Celsius temperature unit"""


class TemporalModifier(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """An optional modifier of a date/time instance."""

    AFTER_APPROXIMATE = "AfterApprox"
    """AfterApprox temporal modifier"""
    BEFORE = "Before"
    """Before temporal modifier"""
    BEFORE_START = "BeforeStart"
    """BeforeStart temporal modifier"""
    APPROXIMATE = "Approx"
    """Approx temporal modifier"""
    REFERENCE_UNDEFINED = "ReferenceUndefined"
    """ReferenceUndefined temporal modifier"""
    SINCE_END = "SinceEnd"
    """SinceEnd temporal modifier"""
    AFTER_MID = "AfterMid"
    """AfterMid temporal modifier"""
    START = "Start"
    """Start temporal modifier"""
    AFTER = "After"
    """After temporal modifier"""
    BEFORE_END = "BeforeEnd"
    """BeforeEnd temporal modifier"""
    UNTIL = "Until"
    """Until temporal modifier"""
    END = "End"
    """End temporal modifier"""
    LESS = "Less"
    """Less temporal modifier"""
    SINCE = "Since"
    """Since temporal modifier"""
    AFTER_START = "AfterStart"
    """AfterStart temporal modifier"""
    BEFORE_APPROXIMATE = "BeforeApprox"
    """BeforeApprox temporal modifier"""
    MID = "Mid"
    """Mid temporal modifier"""
    MORE = "More"
    """More temporal modifier"""


class TranscriptContentType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Enumeration of supported transcript content types."""

    LEXICAL = "lexical"
    """Lexical content type"""
    ITN = "itn"
    """ITN content type"""
    MASKED_ITN = "maskedItn"
    """Masked ITN content type"""
    TEXT = "text"
    """Text content type"""


class VolumeUnit(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The Volume Unit of measurement."""

    UNSPECIFIED = "Unspecified"
    """Unspecified volume unit."""
    CUBIC_METER = "CubicMeter"
    """Volume unit in cubic meters."""
    CUBIC_CENTIMETER = "CubicCentimeter"
    """Volume unit in cubic centimeters."""
    CUBIC_MILLIMETER = "CubicMillimeter"
    """Volume unit in cubic millimeters."""
    HECTOLITER = "Hectoliter"
    """Volume unit in hectoliters."""
    DECALITER = "Decaliter"
    """Volume unit in decaliters."""
    LITER = "Liter"
    """Volume unit in liters."""
    CENTILITER = "Centiliter"
    """Volume unit in centiliters."""
    MILLILITER = "Milliliter"
    """Volume unit in milliliters."""
    CUBIC_YARD = "CubicYard"
    """Volume unit in cubic yards."""
    CUBIC_INCH = "CubicInch"
    """Volume unit in cubic inches."""
    CUBIC_FOOT = "CubicFoot"
    """Volume unit in cubic feet."""
    CUBIC_MILE = "CubicMile"
    """Volume unit in cubic miles."""
    FLUID_OUNCE = "FluidOunce"
    """Volume unit in fluid ounces."""
    TEASPOON = "Teaspoon"
    """Volume unit in teaspoons."""
    TABLESPOON = "Tablespoon"
    """Volume unit in tablespoons."""
    PINT = "Pint"
    """Volume unit in pints."""
    QUART = "Quart"
    """Volume unit in quarts."""
    CUP = "Cup"
    """Volume unit in cups."""
    GILL = "Gill"
    """Volume unit in gills."""
    PINCH = "Pinch"
    """Volume unit in pinches."""
    FLUID_DRAM = "FluidDram"
    """Volume unit in fluid drams."""
    BARREL = "Barrel"
    """Volume unit in barrels."""
    MINIM = "Minim"
    """Volume unit in minims."""
    CORD = "Cord"
    """Volume unit in cords."""
    PECK = "Peck"
    """Volume unit in pecks."""
    BUSHEL = "Bushel"
    """Volume unit in bushels."""
    HOGSHEAD = "Hogshead"
    """Volume unit in hogsheads."""


class WeightUnit(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The weight Unit of measurement."""

    UNSPECIFIED = "Unspecified"
    """Unspecified weight unit"""
    KILOGRAM = "Kilogram"
    """Weight unit in kilograms"""
    GRAM = "Gram"
    """Weight unit in grams"""
    MILLIGRAM = "Milligram"
    """Weight unit in milligrams"""
    GALLON = "Gallon"
    """Volume unit in gallons"""
    METRIC_TON = "MetricTon"
    """Weight unit in metric tons"""
    TON = "Ton"
    """Weight unit in tons"""
    POUND = "Pound"
    """Weight unit in pounds"""
    OUNCE = "Ounce"
    """Weight unit in ounces"""
    GRAIN = "Grain"
    """Weight unit in grains"""
    PENNY_WEIGHT = "PennyWeight"
    """Weight unit in pennyweights"""
    LONG_TON_BRITISH = "LongTonBritish"
    """Weight unit in long tons (British)"""
    SHORT_TON_US = "ShortTonUS"
    """Weight unit in short tons (US)"""
    SHORT_HUNDRED_WEIGHT_US = "ShortHundredWeightUS"
    """Weight unit in short hundredweights (US)"""
    STONE = "Stone"
    """Weight unit in stones"""
    DRAM = "Dram"
    """Weight unit in drams"""



# coding=utf-8
# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------

"""
FILE: sample_conversation_prediction_with_options_async.py

DESCRIPTION:
    This sample demonstrates how to analyze an utterance using a CLU Conversation project (async),
    showing how to set additional options such as `verbose=True`.

USAGE:
    python sample_conversation_prediction_with_options_async.py

REQUIRED ENV VARS (for AAD / DefaultAzureCredential):
    AZURE_CONVERSATIONS_ENDPOINT
    AZURE_CLIENT_ID
    AZURE_TENANT_ID
    AZURE_CLIENT_SECRET
    AZURE_CONVERSATIONS_PROJECT_NAME
    AZURE_CONVERSATIONS_DEPLOYMENT_NAME

NOTE:
    If you want to use AzureKeyCredential instead, set:
      - AZURE_CONVERSATIONS_ENDPOINT
      - AZURE_CONVERSATIONS_KEY
"""

# [START conversation_prediction_with_options_async]
import os
import asyncio

from azure.identity.aio import DefaultAzureCredential
from azure.ai.language.conversations.aio import ConversationAnalysisClient
from azure.ai.language.conversations.models import (
    ConversationLanguageUnderstandingInput,
    ConversationAnalysisInput,
    TextConversationItem,
    ConversationActionContent,
    StringIndexType,
    ConversationActionResult,
    ConversationPrediction,
    DateTimeResolution,
)


async def sample_conversation_prediction_with_options_async():
    # settings
    endpoint = os.environ["AZURE_CONVERSATIONS_ENDPOINT"]
    project_name = os.environ["AZURE_CONVERSATIONS_PROJECT_NAME"]
    deployment_name = os.environ["AZURE_CONVERSATIONS_DEPLOYMENT_NAME"]

    credential = DefaultAzureCredential()

    async with ConversationAnalysisClient(endpoint, credential=credential) as client:
        # build request with verbose=True
        data = ConversationLanguageUnderstandingInput(
            conversation_input=ConversationAnalysisInput(
                conversation_item=TextConversationItem(
                    id="1",
                    participant_id="participant1",
                    text="Send an email to Carol about tomorrow's demo",
                )
            ),
            action_content=ConversationActionContent(
                project_name=project_name,
                deployment_name=deployment_name,
                string_index_type=StringIndexType.UTF16_CODE_UNIT,
                verbose=True,
            ),
        )

        # call async API
        response = await client.analyze_conversation(data)

        if isinstance(response, ConversationActionResult):
            pred = response.result.prediction
            if isinstance(pred, ConversationPrediction):
                # top intent
                print(f"Top intent: {pred.top_intent}\n")

                # intents
                print("Intents:")
                for intent in pred.intents or []:
                    print(f"  Category: {intent.category}")
                    print(f"  Confidence: {intent.confidence}")
                    print()

                # entities
                print("Entities:")
                for entity in pred.entities or []:
                    print(f"  Category: {entity.category}")
                    print(f"  Text: {entity.text}")
                    print(f"  Offset: {entity.offset}")
                    print(f"  Length: {entity.length}")
                    print(f"  Confidence: {entity.confidence}")

                    for res in entity.resolutions or []:
                        if isinstance(res, DateTimeResolution):
                            print("  DateTime Resolution:")
                            print(f"    Sub Kind: {res.date_time_sub_kind}")
                            print(f"    Timex: {res.timex}")
                            print(f"    Value: {res.value}")
                    print()
        else:
            print("Unexpected result type from analyze_conversation.")


# [END conversation_prediction_with_options_async]


async def main():
    await sample_conversation_prediction_with_options_async()


if __name__ == "__main__":
    loop = asyncio.get_event_loop()
    loop.run_until_complete(main())



# pylint: disable=line-too-long,useless-suppression
import functools
import pytest

from devtools_testutils import AzureRecordedTestCase, EnvironmentVariableLoader
from devtools_testutils.aio import recorded_by_proxy_async
from azure.core.async_paging import AsyncItemPaged
from azure.ai.language.conversations.aio import ConversationAnalysisClient, AnalyzeConversationAsyncLROPoller
from azure.ai.language.conversations.models import (
    MultiLanguageConversationInput,
    TextConversation,
    TextConversationItem,
    SummarizationOperationAction,
    ConversationSummarizationActionContent,
    SummaryAspect,
    AnalyzeConversationOperationInput,
    ParticipantRole,
    AnalyzeConversationOperationAction,
    SummarizationOperationResult,
    ConversationActions,
)
from typing import cast, List
from azure.core.credentials import AzureKeyCredential

ConversationsPreparer = functools.partial(
    EnvironmentVariableLoader,
    "conversations",
    conversations_endpoint="https://Sanitized.cognitiveservices.azure.com/",
    conversations_key="fake_key",
)


class TestConversations(AzureRecordedTestCase):

    # Start with any helper functions you might need, for example a client creation method:
    async def create_client(self, endpoint, key):
        credential = AzureKeyCredential(key)
        client = ConversationAnalysisClient(endpoint, credential)
        return client

    ...


class TestConversationsCase(TestConversations):
    @ConversationsPreparer()
    @recorded_by_proxy_async
    @pytest.mark.asyncio
    async def test_conversation_summarization_async(self, conversations_endpoint, conversations_key):
        client = await self.create_client(conversations_endpoint, conversations_key)

        try:
            # Build conversation input (same as sync)
            conversation_items = [
                TextConversationItem(
                    id="1", participant_id="Agent_1", text="Hello, how can I help you?", role=ParticipantRole.AGENT
                ),
                TextConversationItem(
                    id="2",
                    participant_id="Customer_1",
                    text="How to upgrade Office? I am getting error messages the whole day.",
                    role=ParticipantRole.CUSTOMER,
                ),
                TextConversationItem(
                    id="3",
                    participant_id="Agent_1",
                    text="Press the upgrade button please. Then sign in and follow the instructions.",
                    role=ParticipantRole.AGENT,
                ),
            ]
            conversation_input = MultiLanguageConversationInput(
                conversations=[TextConversation(id="1", language="en", conversation_items=conversation_items)]
            )

            actions = [
                SummarizationOperationAction(
                    name="Issue task",
                    action_content=ConversationSummarizationActionContent(summary_aspects=[SummaryAspect.ISSUE]),
                ),
                SummarizationOperationAction(
                    name="Resolution task",
                    action_content=ConversationSummarizationActionContent(summary_aspects=[SummaryAspect.RESOLUTION]),
                ),
            ]

            operation_input = AnalyzeConversationOperationInput(
                conversation_input=conversation_input,
                actions=cast(List[AnalyzeConversationOperationAction], actions),
            )

            # ---- begin_* now returns AnalyzeConversationAsyncLROPoller[AsyncItemPaged[ConversationActions]]
            async with client:
                poller: AnalyzeConversationAsyncLROPoller[AsyncItemPaged[ConversationActions]] = (
                    await client.begin_analyze_conversation_job(body=operation_input, content_type="application/json")
                )

                # Operation id is available immediately
                op_id = poller.details.get("operation_id")
                if op_id:
                    print(f"Operation ID: {op_id}")

                # Result is AsyncItemPaged[ConversationActions]
                paged_actions: AsyncItemPaged[ConversationActions] = await poller.result()

                # Final-state metadata via poller.details
                d = poller.details
                print(f"Job ID: {d.get('job_id')}")
                print(f"Status: {d.get('status')}")
                print(f"Created: {d.get('created_date_time')}")
                print(f"Last Updated: {d.get('last_updated_date_time')}")
                if d.get("expiration_date_time"):
                    print(f"Expires: {d.get('expiration_date_time')}")
                if d.get("display_name"):
                    print(f"Display Name: {d.get('display_name')}")

                # Iterate items (each item is a ConversationActions)
                async for actions_page in paged_actions:
                    print(
                        f"Completed: {actions_page.completed}, "
                        f"In Progress: {actions_page.in_progress}, "
                        f"Failed: {actions_page.failed}, "
                        f"Total: {actions_page.total}"
                    )

                    for action_result in actions_page.task_results or []:
                        print(f"\nAction Name: {getattr(action_result, 'name', None)}")
                        print(f"Action Status: {getattr(action_result, 'status', None)}")
                        print(f"Kind: {getattr(action_result, 'kind', None)}")

                        if isinstance(action_result, SummarizationOperationResult):
                            for conversation in action_result.results.conversations:
                                print(f"  Conversation ID: {conversation.id}")
                                print("  Summaries:")
                                for summary in conversation.summaries:
                                    print(f"    Aspect: {summary.aspect}")
                                    print(f"    Text: {summary.text}")
                                if conversation.warnings:
                                    print("  Warnings:")
                                    for warning in conversation.warnings:
                                        print(f"    Code: {warning.code}, Message: {warning.message}")
                        else:
                            print("  [No supported results to display for this action type]")

                # Print errors (exposed via poller.details from final state)
                if d.get("errors"):
                    print("\nErrors:")
                    for error in d["errors"]:
                        print(f"  Code: {error.code} - {error.message}")

                # Simple sanity checks
                assert d.get("status") in {"succeeded", "partiallyCompleted", "running", "notStarted"}
        finally:
            await client.close()



# coding=utf-8
# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------

"""
FILE: sample_orchestration_prediction_async.py

DESCRIPTION:
    This sample demonstrates how to analyze a user query using an orchestration project.
    In this sample, the orchestration project's top intent routes to a QnA project.

USAGE:
    python sample_orchestration_prediction_async.py

REQUIRED ENV VARS (for AAD / DefaultAzureCredential):
    AZURE_CONVERSATIONS_ENDPOINT
    AZURE_CLIENT_ID
    AZURE_TENANT_ID
    AZURE_CLIENT_SECRET
    AZURE_CONVERSATIONS_PROJECT_NAME
    AZURE_CONVERSATIONS_DEPLOYMENT_NAME

NOTE:
    If you want to use AzureKeyCredential instead, set:
      - AZURE_CONVERSATIONS_ENDPOINT
      - AZURE_CONVERSATIONS_KEY
      - AZURE_CONVERSATIONS_PROJECT_NAME
      - AZURE_CONVERSATIONS_DEPLOYMENT_NAME
"""

# [START orchestration_prediction_async]
import os
import asyncio

from azure.identity.aio import DefaultAzureCredential
from azure.ai.language.conversations.aio import ConversationAnalysisClient
from azure.ai.language.conversations.models import (
    ConversationActionContent,
    ConversationAnalysisInput,
    TextConversationItem,
    StringIndexType,
    ConversationLanguageUnderstandingInput,
    OrchestrationPrediction,
    QuestionAnsweringTargetIntentResult,
    ConversationActionResult,
)


async def sample_orchestration_prediction_async():
    # settings
    endpoint = os.environ["AZURE_CONVERSATIONS_ENDPOINT"]
    project_name = os.environ["AZURE_CONVERSATIONS_PROJECT_NAME"]
    deployment_name = os.environ["AZURE_CONVERSATIONS_DEPLOYMENT_NAME"]

    credential = DefaultAzureCredential()

    async with ConversationAnalysisClient(endpoint, credential=credential) as client:
        # Build request using strongly-typed models
        data = ConversationLanguageUnderstandingInput(
            conversation_input=ConversationAnalysisInput(
                conversation_item=TextConversationItem(
                    id="1",
                    participant_id="participant1",
                    text="How are you?",
                )
            ),
            action_content=ConversationActionContent(
                project_name=project_name,
                deployment_name=deployment_name,
                string_index_type=StringIndexType.UTF16_CODE_UNIT,
            ),
        )

        # Call async API
        response = await client.analyze_conversation(data)

        # Narrow to expected result types
        if isinstance(response, ConversationActionResult):
            pred = response.result.prediction
            if isinstance(pred, OrchestrationPrediction):
                # Top intent name is the routed project name
                top_intent = pred.top_intent
                if not top_intent:
                    print("No top intent was returned by orchestration.")
                    return

                print(f"Top intent (responding project): {top_intent}")

                # Look up the routed target result
                target_intent_result = pred.intents.get(top_intent)
                if not isinstance(target_intent_result, QuestionAnsweringTargetIntentResult):
                    print("Top intent did not route to a Question Answering result.")
                    return

                qa = target_intent_result.result
                if qa is not None and qa.answers is not None:
                    for ans in qa.answers:
                        print(ans.answer or "")
            else:
                print("Prediction was not an OrchestrationPrediction.")
        else:
            print("Unexpected result type from analyze_conversation.")


# [END orchestration_prediction_async]


async def main():
    await sample_orchestration_prediction_async()


if __name__ == "__main__":
    loop = asyncio.get_event_loop()
    loop.run_until_complete(main())



# coding=utf-8
# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------

"""
FILE: sample_orchestration_prediction.py

DESCRIPTION:
    This sample demonstrates how to analyze a user query using an orchestration project.
    In this sample, the orchestration project's top intent routes to a QnA project.

USAGE:
    python sample_orchestration_prediction.py

REQUIRED ENV VARS (for AAD / DefaultAzureCredential):
    AZURE_CONVERSATIONS_ENDPOINT
    AZURE_CLIENT_ID
    AZURE_TENANT_ID
    AZURE_CLIENT_SECRET
    AZURE_CONVERSATIONS_PROJECT_NAME
    AZURE_CONVERSATIONS_DEPLOYMENT_NAME

NOTE:
    If you want to use AzureKeyCredential instead, set:
      - AZURE_CONVERSATIONS_ENDPOINT
      - AZURE_CONVERSATIONS_KEY
      - AZURE_CONVERSATIONS_PROJECT_NAME
      - AZURE_CONVERSATIONS_DEPLOYMENT_NAME
"""

# [START orchestration_prediction]
import os

from azure.identity import DefaultAzureCredential
from azure.ai.language.conversations import ConversationAnalysisClient
from azure.ai.language.conversations.models import (
    ConversationActionContent,
    ConversationAnalysisInput,
    TextConversationItem,
    StringIndexType,
    ConversationLanguageUnderstandingInput,
    OrchestrationPrediction,
    QuestionAnsweringTargetIntentResult,
    ConversationActionResult,
)


def sample_orchestration_prediction():
    # settings
    endpoint = os.environ["AZURE_CONVERSATIONS_ENDPOINT"]
    project_name = os.environ["AZURE_CONVERSATIONS_PROJECT_NAME"]
    deployment_name = os.environ["AZURE_CONVERSATIONS_DEPLOYMENT_NAME"]

    credential = DefaultAzureCredential()
    client = ConversationAnalysisClient(endpoint, credential=credential)

    # Build request using strongly-typed models
    data = ConversationLanguageUnderstandingInput(
        conversation_input=ConversationAnalysisInput(
            conversation_item=TextConversationItem(
                id="1",
                participant_id="participant1",
                text="How are you?",
            )
        ),
        action_content=ConversationActionContent(
            project_name=project_name,
            deployment_name=deployment_name,
            string_index_type=StringIndexType.UTF16_CODE_UNIT,
        ),
    )

    # Call sync API
    response = client.analyze_conversation(data)

    # Narrow to expected result types
    if isinstance(response, ConversationActionResult):
        pred = response.result.prediction
        if isinstance(pred, OrchestrationPrediction):
            # Top intent name is the routed project name
            top_intent = pred.top_intent
            if not top_intent:
                print("No top intent was returned by orchestration.")
                return

            print(f"Top intent (responding project): {top_intent}")

            # Look up the routed target result
            target_intent_result = pred.intents.get(top_intent)
            if not isinstance(target_intent_result, QuestionAnsweringTargetIntentResult):
                print("Top intent did not route to a Question Answering result.")
                return

            qa = target_intent_result.result
            if qa is not None and qa.answers is not None:
                for ans in qa.answers:
                    print(ans.answer or "")
        else:
            print("Prediction was not an OrchestrationPrediction.")
    else:
        print("Unexpected result type from analyze_conversation.")


# [END orchestration_prediction]


def main():
    sample_orchestration_prediction()


if __name__ == "__main__":
    main()



# pylint: disable=line-too-long,useless-suppression
import functools
import pytest

from devtools_testutils import AzureRecordedTestCase, EnvironmentVariableLoader, recorded_by_proxy
from azure.ai.language.conversations.aio import ConversationAnalysisClient
from azure.ai.language.conversations.models import (
    ConversationalAITask,
    ConversationalAIAnalysisInput,
    TextConversation,
    TextConversationItem,
    ConversationalAIActionContent,
    AnalyzeConversationActionResult,
    StringIndexType,
    ConversationalAITaskResult,
    ConversationalAIResult,
    ConversationalAIAnalysis,
    ConversationalAIIntent,
    ConversationalAIEntity,
    ConversationItemRange,
    DateTimeResolution,
    EntitySubtype,
    EntityTag,
)
from typing import cast
from devtools_testutils.aio import recorded_by_proxy_async
from azure.core.async_paging import AsyncItemPaged

from azure.core.credentials import AzureKeyCredential

ConversationsPreparer = functools.partial(
    EnvironmentVariableLoader,
    "conversations",
    conversations_endpoint="https://Sanitized.cognitiveservices.azure.com/",
    conversations_key="fake_key",
)


class TestConversations(AzureRecordedTestCase):

    # Start with any helper functions you might need, for example a client creation method:
    async def create_client(self, endpoint, key):
        credential = AzureKeyCredential(key)
        client = ConversationAnalysisClient(endpoint, credential)
        return client

    ...


class TestConversationsCase(TestConversations):
    @ConversationsPreparer()
    @recorded_by_proxy_async
    @pytest.mark.asyncio
    async def test_conversation_multi_turn_prediction_async(self, conversations_endpoint, conversations_key):
        client = await self.create_client(conversations_endpoint, conversations_key)

        try:
            project_name = "EmailApp"
            deployment_name = "production"

            data = ConversationalAITask(
                analysis_input=ConversationalAIAnalysisInput(
                    conversations=[
                        TextConversation(
                            id="order",
                            language="en-GB",
                            conversation_items=[
                                TextConversationItem(id="1", participant_id="user", text="Hi"),
                                TextConversationItem(id="2", participant_id="bot", text="Hello, how can I help you?"),
                                TextConversationItem(
                                    id="3",
                                    participant_id="user",
                                    text="Send an email to Carol about tomorrow's demo",
                                ),
                            ],
                        )
                    ]
                ),
                parameters=ConversationalAIActionContent(
                    project_name=project_name,
                    deployment_name=deployment_name,
                    string_index_type=StringIndexType.UTF16_CODE_UNIT,
                ),
            )

            # Async call
            response: AnalyzeConversationActionResult = await client.analyze_conversation(data)

            # Cast to ConversationalAI task result
            ai_task_result = cast(ConversationalAITaskResult, response)
            ai_result: ConversationalAIResult = ai_task_result.result

            # Basic sanity
            assert ai_result is not None
            assert ai_result.conversations is not None

            # Iterate conversations
            for conversation in ai_result.conversations or []:
                conversation = cast(ConversationalAIAnalysis, conversation)
                print(f"Conversation ID: {conversation.id}\n")

                # Intents
                print("Intents:")
                for intent in conversation.intents or []:
                    intent = cast(ConversationalAIIntent, intent)
                    print(f"  Name: {intent.name}")
                    print(f"  Type: {getattr(intent, 'type', None)}")

                    print("  Conversation Item Ranges:")
                    for rng in intent.conversation_item_ranges or []:
                        rng = cast(ConversationItemRange, rng)
                        print(f"    - Offset: {rng.offset}, Count: {rng.count}")

                    print("\n  Entities (Scoped to Intent):")
                    for ent in intent.entities or []:
                        ent = cast(ConversationalAIEntity, ent)
                        print(f"    Name: {ent.name}")
                        print(f"    Text: {ent.text}")
                        print(f"    Confidence: {ent.confidence_score}")
                        print(f"    Offset: {ent.offset}, Length: {ent.length}")
                        print(
                            f"    Conversation Item ID: {ent.conversation_item_id}, "
                            f"Index: {ent.conversation_item_index}"
                        )

                        # Date/time resolutions
                        if ent.resolutions:
                            for res in ent.resolutions:
                                if isinstance(res, DateTimeResolution):
                                    print(
                                        f"    - [DateTimeResolution] SubKind: {getattr(res, 'date_time_sub_kind', None)}, "
                                        f"Timex: {res.timex}, Value: {res.value}"
                                    )

                        # Extra information (entity subtype + tags)
                        if ent.extra_information:
                            for extra in ent.extra_information:
                                if isinstance(extra, EntitySubtype):
                                    print(f"    - [EntitySubtype] Value: {extra.value}")
                                    for tag in extra.tags or []:
                                        tag = cast(EntityTag, tag)
                                        print(f"      • Tag: {tag.name}, Confidence: {tag.confidence_score}")

                    print()

                # Global entities
                print("Global Entities:")
                for ent in conversation.entities or []:
                    ent = cast(ConversationalAIEntity, ent)
                    print(f"  Name: {ent.name}")
                    print(f"  Text: {ent.text}")
                    print(f"  Confidence: {ent.confidence_score}")
                    print(f"  Offset: {ent.offset}, Length: {ent.length}")
                    print(
                        f"  Conversation Item ID: {ent.conversation_item_id}, " f"Index: {ent.conversation_item_index}"
                    )

                    if ent.extra_information:
                        for extra in ent.extra_information:
                            if isinstance(extra, EntitySubtype):
                                print(f"    - [EntitySubtype] Value: {extra.value}")
                                for tag in extra.tags or []:
                                    tag = cast(EntityTag, tag)
                                    print(f"      • Tag: {tag.name}, Confidence: {tag.confidence_score}")
                print("-" * 40)
        finally:
            await client.close()



# pylint: disable=line-too-long,useless-suppression
# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
from collections.abc import MutableMapping # pylint:disable=import-error
from io import IOBase
import json
from typing import Any, AsyncIterator, Callable, IO, Optional, TypeVar, Union, cast, overload

from azure.core import AsyncPipelineClient
from azure.core.exceptions import (
    ClientAuthenticationError,
    HttpResponseError,
    ResourceExistsError,
    ResourceNotFoundError,
    ResourceNotModifiedError,
    StreamClosedError,
    StreamConsumedError,
    map_error,
)
from azure.core.pipeline import PipelineResponse
from azure.core.polling import AsyncLROPoller, AsyncNoPolling, AsyncPollingMethod
from azure.core.polling.async_base_polling import AsyncLROBasePolling
from azure.core.rest import AsyncHttpResponse, HttpRequest
from azure.core.tracing.decorator_async import distributed_trace_async
from azure.core.utils import case_insensitive_dict

from ... import models as _models
from ..._operations._operations import (
    build_conversation_analysis_analyze_conversation_job_request,
    build_conversation_analysis_analyze_conversation_request,
    build_conversation_analysis_cancel_job_request,
    build_conversation_analysis_get_job_status_request,
)
from ..._utils.model_base import SdkJSONEncoder, _deserialize, _failsafe_deserialize
from ..._utils.utils import ClientMixinABC
from ..._validation import api_version_validation
from .._configuration import ConversationAnalysisClientConfiguration

JSON = MutableMapping[str, Any]
T = TypeVar("T")
ClsType = Optional[Callable[[PipelineResponse[HttpRequest, AsyncHttpResponse], T, dict[str, Any]], Any]]


class _ConversationAnalysisClientOperationsMixin(
    ClientMixinABC[AsyncPipelineClient[HttpRequest, AsyncHttpResponse], ConversationAnalysisClientConfiguration]
):

    @overload
    async def analyze_conversation(
        self, body: _models.AnalyzeConversationInput, *, content_type: str = "application/json", **kwargs: Any
    ) -> _models.AnalyzeConversationActionResult:
        """Analyzes the input conversation utterance.

        :param body: The input for the analyze conversations operation. Required.
        :type body: ~azure.ai.language.conversations.models.AnalyzeConversationInput
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: AnalyzeConversationActionResult. The AnalyzeConversationActionResult is compatible
         with MutableMapping
        :rtype: ~azure.ai.language.conversations.models.AnalyzeConversationActionResult
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    async def analyze_conversation(
        self, body: JSON, *, content_type: str = "application/json", **kwargs: Any
    ) -> _models.AnalyzeConversationActionResult:
        """Analyzes the input conversation utterance.

        :param body: The input for the analyze conversations operation. Required.
        :type body: JSON
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: AnalyzeConversationActionResult. The AnalyzeConversationActionResult is compatible
         with MutableMapping
        :rtype: ~azure.ai.language.conversations.models.AnalyzeConversationActionResult
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    async def analyze_conversation(
        self, body: IO[bytes], *, content_type: str = "application/json", **kwargs: Any
    ) -> _models.AnalyzeConversationActionResult:
        """Analyzes the input conversation utterance.

        :param body: The input for the analyze conversations operation. Required.
        :type body: IO[bytes]
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: AnalyzeConversationActionResult. The AnalyzeConversationActionResult is compatible
         with MutableMapping
        :rtype: ~azure.ai.language.conversations.models.AnalyzeConversationActionResult
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace_async
    async def analyze_conversation(
        self, body: Union[_models.AnalyzeConversationInput, JSON, IO[bytes]], **kwargs: Any
    ) -> _models.AnalyzeConversationActionResult:
        """Analyzes the input conversation utterance.

        :param body: The input for the analyze conversations operation. Is one of the following types:
         AnalyzeConversationInput, JSON, IO[bytes] Required.
        :type body: ~azure.ai.language.conversations.models.AnalyzeConversationInput or JSON or
         IO[bytes]
        :return: AnalyzeConversationActionResult. The AnalyzeConversationActionResult is compatible
         with MutableMapping
        :rtype: ~azure.ai.language.conversations.models.AnalyzeConversationActionResult
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[_models.AnalyzeConversationActionResult] = kwargs.pop("cls", None)

        content_type = content_type or "application/json"
        _content = None
        if isinstance(body, (IOBase, bytes)):
            _content = body
        else:
            _content = json.dumps(body, cls=SdkJSONEncoder, exclude_readonly=True)  # type: ignore

        _request = build_conversation_analysis_analyze_conversation_request(
            content_type=content_type,
            api_version=self._config.api_version,
            content=_content,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "Endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _stream = kwargs.pop("stream", False)
        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            if _stream:
                try:
                    await response.read()  # Load the body in memory and close the socket
                except (StreamConsumedError, StreamClosedError):
                    pass
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = _failsafe_deserialize(
                _models.ErrorResponse,
                response,
            )
            raise HttpResponseError(response=response, model=error)

        if _stream:
            deserialized = response.iter_bytes()
        else:
            deserialized = _deserialize(_models.AnalyzeConversationActionResult, response.json())

        if cls:
            return cls(pipeline_response, deserialized, {})  # type: ignore

        return deserialized  # type: ignore

    @distributed_trace_async
    @api_version_validation(
        method_added_on="2023-04-01",
        params_added_on={"2023-04-01": ["api_version", "job_id", "show_stats", "accept"]},
        api_versions_list=["2023-04-01", "2024-05-01", "2024-11-01", "2025-05-15-preview", "2025-11-15-preview"],
    )
    async def _get_job_status(
        self, job_id: str, *, show_stats: Optional[bool] = None, **kwargs: Any
    ) -> _models.AnalyzeConversationOperationState:
        """Get analysis status and results.

        Get the status of an analysis job. A job can consist of one or more tasks. After all tasks
        succeed, the job transitions to the succeeded state and results are available for each task.

        :param job_id: job ID. Required.
        :type job_id: str
        :keyword show_stats: (Optional) if set to true, response will contain request and document
         level statistics. Default value is None.
        :paramtype show_stats: bool
        :return: AnalyzeConversationOperationState. The AnalyzeConversationOperationState is compatible
         with MutableMapping
        :rtype: ~azure.ai.language.conversations.models.AnalyzeConversationOperationState
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls: ClsType[_models.AnalyzeConversationOperationState] = kwargs.pop("cls", None)

        _request = build_conversation_analysis_get_job_status_request(
            job_id=job_id,
            show_stats=show_stats,
            api_version=self._config.api_version,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "Endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _stream = kwargs.pop("stream", False)
        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            if _stream:
                try:
                    await response.read()  # Load the body in memory and close the socket
                except (StreamConsumedError, StreamClosedError):
                    pass
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = _failsafe_deserialize(
                _models.ErrorResponse,
                response,
            )
            raise HttpResponseError(response=response, model=error)

        if _stream:
            deserialized = response.iter_bytes()
        else:
            deserialized = _deserialize(_models.AnalyzeConversationOperationState, response.json())

        if cls:
            return cls(pipeline_response, deserialized, {})  # type: ignore

        return deserialized  # type: ignore

    @api_version_validation(
        method_added_on="2023-04-01",
        params_added_on={"2023-04-01": ["api_version", "content_type"]},
        api_versions_list=["2023-04-01", "2024-05-01", "2024-11-01", "2025-05-15-preview", "2025-11-15-preview"],
    )
    async def _analyze_conversation_job_initial(
        self, body: Union[_models._models.AnalyzeConversationOperationInput, JSON, IO[bytes]], **kwargs: Any
    ) -> AsyncIterator[bytes]:
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[AsyncIterator[bytes]] = kwargs.pop("cls", None)

        content_type = content_type or "application/json"
        _content = None
        if isinstance(body, (IOBase, bytes)):
            _content = body
        else:
            _content = json.dumps(body, cls=SdkJSONEncoder, exclude_readonly=True)  # type: ignore

        _request = build_conversation_analysis_analyze_conversation_job_request(
            content_type=content_type,
            api_version=self._config.api_version,
            content=_content,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "Endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _stream = True
        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [202]:
            try:
                await response.read()  # Load the body in memory and close the socket
            except (StreamConsumedError, StreamClosedError):
                pass
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = _failsafe_deserialize(
                _models.ErrorResponse,
                response,
            )
            raise HttpResponseError(response=response, model=error)

        response_headers = {}
        response_headers["Operation-Location"] = self._deserialize("str", response.headers.get("Operation-Location"))

        deserialized = response.iter_bytes()

        if cls:
            return cls(pipeline_response, deserialized, response_headers)  # type: ignore

        return deserialized  # type: ignore

    @overload
    async def _begin_analyze_conversation_job(
        self,
        body: _models._models.AnalyzeConversationOperationInput,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> AsyncLROPoller[None]: ...
    @overload
    async def _begin_analyze_conversation_job(
        self, body: JSON, *, content_type: str = "application/json", **kwargs: Any
    ) -> AsyncLROPoller[None]: ...
    @overload
    async def _begin_analyze_conversation_job(
        self, body: IO[bytes], *, content_type: str = "application/json", **kwargs: Any
    ) -> AsyncLROPoller[None]: ...

    @distributed_trace_async
    @api_version_validation(
        method_added_on="2023-04-01",
        params_added_on={"2023-04-01": ["api_version", "content_type"]},
        api_versions_list=["2023-04-01", "2024-05-01", "2024-11-01", "2025-05-15-preview", "2025-11-15-preview"],
    )
    async def _begin_analyze_conversation_job(
        self, body: Union[_models._models.AnalyzeConversationOperationInput, JSON, IO[bytes]], **kwargs: Any
    ) -> AsyncLROPoller[None]:
        """Analyzes the input conversation utterance.

        :param body: The input for the analyze conversations operation. Is one of the following types:
         AnalyzeConversationOperationInput, JSON, IO[bytes] Required.
        :type body: ~azure.ai.language.conversations.models._models.AnalyzeConversationOperationInput
         or JSON or IO[bytes]
        :return: An instance of AsyncLROPoller that returns None
        :rtype: ~azure.core.polling.AsyncLROPoller[None]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[None] = kwargs.pop("cls", None)
        polling: Union[bool, AsyncPollingMethod] = kwargs.pop("polling", True)
        lro_delay = kwargs.pop("polling_interval", self._config.polling_interval)
        cont_token: Optional[str] = kwargs.pop("continuation_token", None)
        if cont_token is None:
            raw_result = await self._analyze_conversation_job_initial(
                body=body, content_type=content_type, cls=lambda x, y, z: x, headers=_headers, params=_params, **kwargs
            )
            await raw_result.http_response.read()  # type: ignore
        kwargs.pop("error_map", None)

        def get_long_running_output(pipeline_response):  # pylint: disable=inconsistent-return-statements
            if cls:
                return cls(pipeline_response, None, {})  # type: ignore

        path_format_arguments = {
            "Endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }

        if polling is True:
            polling_method: AsyncPollingMethod = cast(
                AsyncPollingMethod,
                AsyncLROBasePolling(lro_delay, path_format_arguments=path_format_arguments, **kwargs),
            )
        elif polling is False:
            polling_method = cast(AsyncPollingMethod, AsyncNoPolling())
        else:
            polling_method = polling
        if cont_token:
            return AsyncLROPoller[None].from_continuation_token(
                polling_method=polling_method,
                continuation_token=cont_token,
                client=self._client,
                deserialization_callback=get_long_running_output,
            )
        return AsyncLROPoller[None](self._client, raw_result, get_long_running_output, polling_method)  # type: ignore

    @api_version_validation(
        method_added_on="2023-04-01",
        params_added_on={"2023-04-01": ["api_version", "job_id"]},
        api_versions_list=["2023-04-01", "2024-05-01", "2024-11-01", "2025-05-15-preview", "2025-11-15-preview"],
    )
    async def _cancel_job_initial(self, job_id: str, **kwargs: Any) -> AsyncIterator[bytes]:
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls: ClsType[AsyncIterator[bytes]] = kwargs.pop("cls", None)

        _request = build_conversation_analysis_cancel_job_request(
            job_id=job_id,
            api_version=self._config.api_version,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "Endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _stream = True
        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [202]:
            try:
                await response.read()  # Load the body in memory and close the socket
            except (StreamConsumedError, StreamClosedError):
                pass
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = _failsafe_deserialize(
                _models.ErrorResponse,
                response,
            )
            raise HttpResponseError(response=response, model=error)

        response_headers = {}
        response_headers["Operation-Location"] = self._deserialize("str", response.headers.get("Operation-Location"))

        deserialized = response.iter_bytes()

        if cls:
            return cls(pipeline_response, deserialized, response_headers)  # type: ignore

        return deserialized  # type: ignore

    @distributed_trace_async
    @api_version_validation(
        method_added_on="2023-04-01",
        params_added_on={"2023-04-01": ["api_version", "job_id"]},
        api_versions_list=["2023-04-01", "2024-05-01", "2024-11-01", "2025-05-15-preview", "2025-11-15-preview"],
    )
    async def begin_cancel_job(self, job_id: str, **kwargs: Any) -> AsyncLROPoller[None]:
        """Cancel a long-running Text Analysis conversations job.

        Cancel a long-running job for text analysis of conversations.

        :param job_id: The job ID to cancel. Required.
        :type job_id: str
        :return: An instance of AsyncLROPoller that returns None
        :rtype: ~azure.core.polling.AsyncLROPoller[None]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls: ClsType[None] = kwargs.pop("cls", None)
        polling: Union[bool, AsyncPollingMethod] = kwargs.pop("polling", True)
        lro_delay = kwargs.pop("polling_interval", self._config.polling_interval)
        cont_token: Optional[str] = kwargs.pop("continuation_token", None)
        if cont_token is None:
            raw_result = await self._cancel_job_initial(
                job_id=job_id, cls=lambda x, y, z: x, headers=_headers, params=_params, **kwargs
            )
            await raw_result.http_response.read()  # type: ignore
        kwargs.pop("error_map", None)

        def get_long_running_output(pipeline_response):  # pylint: disable=inconsistent-return-statements
            if cls:
                return cls(pipeline_response, None, {})  # type: ignore

        path_format_arguments = {
            "Endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }

        if polling is True:
            polling_method: AsyncPollingMethod = cast(
                AsyncPollingMethod,
                AsyncLROBasePolling(lro_delay, path_format_arguments=path_format_arguments, **kwargs),
            )
        elif polling is False:
            polling_method = cast(AsyncPollingMethod, AsyncNoPolling())
        else:
            polling_method = polling
        if cont_token:
            return AsyncLROPoller[None].from_continuation_token(
                polling_method=polling_method,
                continuation_token=cont_token,
                client=self._client,
                deserialization_callback=get_long_running_output,
            )
        return AsyncLROPoller[None](self._client, raw_result, get_long_running_output, polling_method)  # type: ignore



import functools
import pytest

from devtools_testutils import AzureRecordedTestCase, EnvironmentVariableLoader, recorded_by_proxy
from azure.ai.language.conversations import ConversationAnalysisClient
from azure.ai.language.conversations.models import (
    AnalyzeConversationActionResult,
    ConversationActionContent,
    ConversationAnalysisInput,
    TextConversationItem,
    StringIndexType,
    ConversationLanguageUnderstandingInput,
    OrchestrationPrediction,
    QuestionAnsweringTargetIntentResult,
    ConversationActionResult,
)
from typing import cast

from azure.core.credentials import AzureKeyCredential

ConversationsPreparer = functools.partial(
    EnvironmentVariableLoader,
    "conversations",
    conversations_endpoint="https://Sanitized.cognitiveservices.azure.com/",
    conversations_key="fake_key",
)


class TestConversations(AzureRecordedTestCase):

    # Start with any helper functions you might need, for example a client creation method:
    def create_client(self, endpoint, key):
        credential = AzureKeyCredential(key)
        client = ConversationAnalysisClient(endpoint, credential)
        return client

    ...


class TestConversationsCase(TestConversations):
    @ConversationsPreparer()
    @recorded_by_proxy
    def test_orchestration_prediction(self, conversations_endpoint, conversations_key):
        client = self.create_client(conversations_endpoint, conversations_key)

        project_name = "TestWorkflow"
        deployment_name = "production"

        # Build request using strongly-typed models
        data = ConversationLanguageUnderstandingInput(
            conversation_input=ConversationAnalysisInput(
                conversation_item=TextConversationItem(
                    id="1",
                    participant_id="participant1",
                    text="How are you?",
                )
            ),
            action_content=ConversationActionContent(
                project_name=project_name,
                deployment_name=deployment_name,
                string_index_type=StringIndexType.UTF16_CODE_UNIT,
            ),
        )

        response: AnalyzeConversationActionResult = client.analyze_conversation(data)

        conversation_result = cast(ConversationActionResult, response)
        prediction = conversation_result.result.prediction

        assert isinstance(prediction, OrchestrationPrediction)

        assert prediction.top_intent is not None, "top_intent missing in orchestration prediction"
        responding_project_name = cast(str, prediction.top_intent)
        print(f"Top intent: {responding_project_name}")

        target_intent_result = prediction.intents[responding_project_name]
        assert isinstance(target_intent_result, QuestionAnsweringTargetIntentResult)

        qa = target_intent_result.result
        for ans in qa.answers:  # type: ignore
            print(ans.answer or "")

        # Final assertions like in C#
        assert responding_project_name == "ChitChat-QnA"
